## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import InputOutputs
import Tables
import Settings
import Universals
import Dialogs
from MyObjects import *
import ReportBug
import Organizer
import Databases

class FileManager():
    
    def __init__(self, _parent):
        self.bookmarks = Bookmarks(self)
        self.history = []
        self.future = []
        Universals.MainWindow.DirOperator = None
        Universals.MainWindow.Browser = None
        Universals.MainWindow.Places = None
        Universals.MainWindow.TreeBrowser = None
        if Universals.isActivePyKDE4==True:
            self.dirModelMain = MDirModel()
            self.dirLister = MDirLister()
            self.dirLister.setAutoUpdate(True)
            self.dirModelMain.setDirLister(self.dirLister)
            self.dirModel = MDirSortFilterProxyModel()
            self.dirModel.setSourceModel(self.dirModelMain)
            self.fpmPlacesModel = MFilePlacesModel()
            self.fpvPlaces = MFilePlacesView()
            self.fpvPlaces.setModel(self.fpmPlacesModel)
            self.dirModelForNavigator = MDirSortFilterProxyModel()
            self.dirModelForNavigator.setSourceModel(self.dirModelMain)
        else:
            self.dirModel = MDirModel()
            self.dirModel.setFilter(MDir.AllEntries | MDir.Readable | MDir.NoDotAndDotDot)
            self.dirModel.sort(0)
            self.dirModelForNavigator = MDirModel()
            self.dirModelForNavigator.setFilter(MDir.AllEntries | MDir.Readable)
            self.dirModelForNavigator.sort(0)
        self.trvFileManager = MTreeView()
        self.lstvFileManager = MListView()
        self.trvFileManager.setModel(self.dirModel)
        self.lstvFileManager.setModel(self.dirModel)
        self.currentDirectory = trForM(InputOutputs.IA.getRealDirName(Universals.MySettings["lastDirectory"]))
        if InputOutputs.IA.isDir(str(self.currentDirectory))==False:
            self.currentDirectory = MDir.homePath()
        MObject.connect(self.trvFileManager, SIGNAL("clicked(QModelIndex)"),self.setMyCurrentIndex)
        MObject.connect(self.lstvFileManager, SIGNAL("doubleClicked(QModelIndex)"),self.setMyCurrentIndex)
        tools = MToolBar(_parent)
        actAddBookmark = MAction(MIcon("Images:addBookmark.png"),"",tools)
        actAddBookmark.setToolTip(translate("FileManager", "Add To Bookmarks"))
        self.actBack = MAction(MIcon("Images:back.png"),"",tools)
        self.actBack.setToolTip(translate("FileManager", "Back"))
        self.actForward = MAction(MIcon("Images:forward.png"),"",tools)
        self.actForward.setToolTip(translate("FileManager", "Forward"))
        self.actUp = MAction(MIcon("Images:up.png"),"",tools)
        self.actUp.setToolTip(translate("FileManager", "Up"))
        actRefresh = MAction(MIcon("Images:refresh.png"),"",tools)
        actRefresh.setToolTip(translate("FileManager", "Refresh"))
        actHome = MAction(MIcon("Images:home.png"),"",tools)
        actHome.setToolTip(translate("FileManager", "Home"))
        MObject.connect(self.actBack, SIGNAL("triggered(bool)"), self.goBack)
        MObject.connect(self.actForward, SIGNAL("triggered(bool)"), self.goForward)
        MObject.connect(self.actUp, SIGNAL("triggered(bool)"), self.goUp)
        MObject.connect(actRefresh, SIGNAL("triggered(bool)"), self.makeRefresh)
        MObject.connect(actHome, SIGNAL("triggered(bool)"), self.goHome)
        MObject.connect(actAddBookmark, SIGNAL("triggered(bool)"), self.bookmarks.addBookmark)
        self.bookmarksMenu = BookmarksMenu(self)
        if Universals.isActivePyKDE4==True:
            toolsFull = MToolBar(_parent)
            toolsFull.addAction(self.bookmarksMenu.menuAction())
            self.isGoToFromDirOperator = False
            self.dirOperator = MDirOperator(MUrl( self.currentDirectory ), _parent)
            self.dirOperator.setDirLister(self.dirLister)
            
            kconf = MGlobal.config()
            kconfGroup = MConfigGroup(kconf,"DirectoryOperator")
            self.dirOperator.readConfig(kconfGroup)
            self.dirOperator.setViewConfig(kconfGroup)
            self.dirOperator.setView(MFile.Default)
            
            self.actCollection = self.dirOperator.actionCollection()
            self.actCollection.readSettings(kconfGroup)
            self.actCollection.associateWidget(toolsFull)
            
            Universals.MainWindow.DirOperator = MDockWidget(translate("FileManager", "Directory Operator"))
            Universals.MainWindow.DirOperator.setObjectName(translate("FileManager", "Directory Operator"))
            Universals.MainWindow.DirOperator.setWidget(self.dirOperator)
            Universals.MainWindow.DirOperator.setAllowedAreas(Mt.AllDockWidgetAreas)
            Universals.MainWindow.DirOperator.setFeatures(MDockWidget.AllDockWidgetFeatures)
            _parent.addDockWidget(Mt.LeftDockWidgetArea, Universals.MainWindow.DirOperator)
            MObject.connect(self.dirOperator, SIGNAL("urlEntered(KUrl)"),self.dirOperatorUrlChanged)
            self.isGoToFromDirOperator = True
            self.filePlacesModel = MFilePlacesModel()
            self.urlNavigator = MUrlNavigator(self.filePlacesModel, MUrl(self.currentDirectory), self.lstvFileManager)
            self.isGoToFromUrlNavigator = True
            MObject.connect(self.urlNavigator, SIGNAL("urlChanged(KUrl)"),self.urlNavigatorUrlChanged)
            MObject.connect(self.fpvPlaces, SIGNAL("urlChanged(KUrl)"),self.setPlacesUrlChanged)
        self.leNavigator = MLineEdit()
        cmpCompleter = MCompleter()
        cmpCompleter.setModel(self.dirModelForNavigator)
        cmpCompleter.setCaseSensitivity(Mt.CaseInsensitive)
        self.leNavigator.setCompleter(cmpCompleter)
        MObject.connect(self.leNavigator, SIGNAL("returnPressed()"),self.leNavigatorPressed)
        tools.addAction(self.actBack)
        tools.addAction(self.actForward)
        tools.addAction(self.actUp)
        tools.addAction(actRefresh)
        tools.addAction(actHome)
        tools.addAction(self.bookmarksMenu.menuAction())
        tools.addAction(actAddBookmark)
        self.actBack.setEnabled(False)
        self.actForward.setEnabled(False)
        self.goTo(self.currentDirectory)
        if Universals.windowMode==Variables.windowModeKeys[1]:
            tools.setIconSize(MSize(16, 16))
            self.tbarBrowserTools = MToolBar(_parent)
            self.tbarBrowserTools.setWindowTitle(translate("FileManager", "Browser Tools"))
            self.tbarBrowserTools.setObjectName(translate("FileManager", "Browser Tools"))
            self.tbarBrowserTools.setIconSize(MSize(16,16))
            self.tbarBrowserTools.addWidget(tools)
            _parent.addToolBar(Mt.TopToolBarArea, self.tbarBrowserTools)
            self.tbarLocationBar = MToolBar(_parent)
            self.tbarLocationBar.setWindowTitle(translate("FileManager", "Location Bar"))
            self.tbarLocationBar.setObjectName(translate("FileManager", "Location Bar"))
            self.tbarLocationBar.setIconSize(MSize(16,16))
            self.tbarLocationBar.addWidget(self.leNavigator)
            _parent.addToolBar(Mt.TopToolBarArea, self.tbarLocationBar)
            if Universals.isActivePyKDE4==True:
                toolsFull.setIconSize(MSize(16, 16))
                self.tbarBrowserToolsFull = MToolBar(_parent)
                self.tbarBrowserToolsFull.setWindowTitle(translate("FileManager", "Browser Tools (KDE4)"))
                self.tbarBrowserToolsFull.setObjectName(translate("FileManager", "Browser Tools (KDE4)"))
                self.tbarBrowserToolsFull.setIconSize(MSize(16,16))
                self.tbarBrowserToolsFull.addWidget(toolsFull)
                _parent.addToolBar(Mt.TopToolBarArea, self.tbarBrowserToolsFull)
                self.tbarLocationBar = MToolBar(_parent)
                self.tbarLocationBar.setWindowTitle(translate("FileManager", "Location Bar (KDE4)"))
                self.tbarLocationBar.setObjectName(translate("FileManager", "Location Bar (KDE4)"))
                self.tbarLocationBar.setIconSize(MSize(16,16))
                self.tbarLocationBar.addWidget(self.urlNavigator)
                _parent.addToolBar(Mt.TopToolBarArea, self.tbarLocationBar)
        else:
            tools.setIconSize(MSize(22, 22))
            self.dckwBrowserTools = MDockWidget(translate("FileManager", "Browser Tools"))
            self.dckwBrowserTools.setObjectName(translate("FileManager", "Browser Tools"))
            self.dckwBrowserTools.setWidget(tools)
            self.dckwBrowserTools.setAllowedAreas(Mt.AllDockWidgetAreas)
            self.dckwBrowserTools.setFeatures(MDockWidget.AllDockWidgetFeatures)
            _parent.addDockWidget(Mt.LeftDockWidgetArea, self.dckwBrowserTools)
            self.tbarLocationBar = MToolBar(_parent)
            self.tbarLocationBar.setWindowTitle(translate("FileManager", "Location Bar"))
            self.tbarLocationBar.setObjectName(translate("FileManager", "Location Bar"))
            self.tbarLocationBar.setIconSize(MSize(16,16))
            self.tbarLocationBar.addWidget(self.leNavigator)
            _parent.addToolBar(Mt.TopToolBarArea, self.tbarLocationBar)
            if Universals.isActivePyKDE4==True:
                toolsFull.setIconSize(MSize(22, 22))
                self.dckwBrowserToolsFull = MDockWidget(translate("FileManager", "Browser Tools (KDE4)"))
                self.dckwBrowserToolsFull.setObjectName(translate("FileManager", "Browser Tools (KDE4)"))
                self.dckwBrowserToolsFull.setWidget(toolsFull)
                self.dckwBrowserToolsFull.setAllowedAreas(Mt.AllDockWidgetAreas)
                self.dckwBrowserToolsFull.setFeatures(MDockWidget.AllDockWidgetFeatures)
                _parent.addDockWidget(Mt.LeftDockWidgetArea, self.dckwBrowserToolsFull)
                self.tbarLocationBar = MToolBar(_parent)
                self.tbarLocationBar.setWindowTitle(translate("FileManager", "Location Bar (KDE4)"))
                self.tbarLocationBar.setObjectName(translate("FileManager", "Location Bar (KDE4)"))
                self.tbarLocationBar.setIconSize(MSize(16,16))
                self.tbarLocationBar.addWidget(self.urlNavigator)
                _parent.addToolBar(Mt.TopToolBarArea, self.tbarLocationBar)
        Universals.MainWindow.Browser = MDockWidget(translate("FileManager", "Browser"))
        Universals.MainWindow.Browser.setObjectName(translate("FileManager", "Browser"))
        Universals.MainWindow.Browser.setWidget(self.lstvFileManager)
        Universals.MainWindow.Browser.setAllowedAreas(Mt.AllDockWidgetAreas)
        Universals.MainWindow.Browser.setFeatures(MDockWidget.AllDockWidgetFeatures)
        _parent.addDockWidget(Mt.LeftDockWidgetArea, Universals.MainWindow.Browser)
        Universals.MainWindow.TreeBrowser = MDockWidget(translate("FileManager", "Tree Browser"))
        Universals.MainWindow.TreeBrowser.setObjectName(translate("FileManager", "Tree Browser"))
        Universals.MainWindow.TreeBrowser.setWidget(self.trvFileManager)
        Universals.MainWindow.TreeBrowser.setAllowedAreas(Mt.AllDockWidgetAreas)
        Universals.MainWindow.TreeBrowser.setFeatures(MDockWidget.AllDockWidgetFeatures)
        _parent.addDockWidget(Mt.LeftDockWidgetArea, Universals.MainWindow.TreeBrowser)
        if Universals.isActivePyKDE4==True:
            Universals.MainWindow.Places = MDockWidget(translate("FileManager", "Places"))
            Universals.MainWindow.Places.setObjectName(translate("FileManager", "Places"))
            Universals.MainWindow.Places.setWidget(self.fpvPlaces)
            Universals.MainWindow.Places.setAllowedAreas(Mt.AllDockWidgetAreas)
            Universals.MainWindow.Places.setFeatures(MDockWidget.AllDockWidgetFeatures)
            _parent.addDockWidget(Mt.LeftDockWidgetArea, Universals.MainWindow.Places)
            _parent.tabifyDockWidget(Universals.MainWindow.DirOperator, Universals.MainWindow.Places)
            _parent.tabifyDockWidget(Universals.MainWindow.DirOperator, Universals.MainWindow.TreeBrowser)
            _parent.tabifyDockWidget(Universals.MainWindow.DirOperator, Universals.MainWindow.Browser)

    def goTo(self, _path, _isRemember = True):
        try:
            _path = InputOutputs.getRealPath(str(_path))
            if InputOutputs.IA.isReadableFileOrDir(_path):
                if InputOutputs.IA.isDir(_path):
                    if _isRemember:
                        self.future = []
                        self.history.append(self.currentDirectory)
                    if _path[-1]=="/": _path = _path[:-1]
                    self.currentDirectory = trForM(_path)
                    if Universals.isActivePyKDE4==True:
                        self.dirLister.openUrl(MUrl(self.currentDirectory))
                        self.isGoToFromUrlNavigator = False
                        self.urlNavigator.setUrl(MUrl(self.currentDirectory))
                        self.isGoToFromUrlNavigator = True
                        self.isGoToFromDirOperator = False
                        self.dirOperator.setUrl(MUrl(self.currentDirectory), False)
                        self.isGoToFromDirOperator = True
                    else:
                        self.lstvFileManager.setRootIndex(self.dirModel.index(_path))
                        self.trvFileManager.setCurrentIndex(self.dirModel.index(_path))
                    self.actForward.setEnabled(False)
                    self.showInTable()
                    self.actBack.setEnabled(True)
                    if str(self.currentDirectory)=="/":
                        self.actUp.setEnabled(False)
                    else:
                        self.actUp.setEnabled(True)
                elif InputOutputs.IA.isFile(_path):
                    isOpened = False
                    for ext in Universals.getListFromStrint(Universals.MySettings["musicExtensions"]):
                        if str(_path).split(".")[-1].lower() == str(ext).lower():
                            if Universals.tableType==2 and Universals.MainWindow.PlayerBar.Player.playInBar.isChecked():
                                Universals.MainWindow.PlayerBar.Player.play(str(_path))
                            else:
                                from Details import MusicDetails
                                MusicDetails.MusicDetails(str(_path),Universals.MainWindow.Table.isOpenDetailsOnNewWindow.isChecked())
                            isOpened = True
                    if isOpened==False:
                        try:
                            from Details import TextDetails
                            TextDetails.TextDetails(str(_path),Universals.MainWindow.Table.isOpenDetailsOnNewWindow.isChecked())
                        except:
                            Dialogs.showError(translate("FileManager", "Cannot open file"), 
                                         str(translate("FileManager", "\"%s\" cannot be opened. Please make sure you selected a text file.")) % Organizer.getLink(str(_path)))
        except:
            error = ReportBug.ReportBug()
            error.show()

    def goBack(self):
        try:
            self.future.append(self.currentDirectory)
            self.currentDirectory = self.history.pop()
            self.goTo(self.currentDirectory, False)
            self.actForward.setEnabled(True)
            if len(self.history)==0:
                self.actBack.setEnabled(False)
        except:
            error = ReportBug.ReportBug()
            error.show()

    def goForward(self):
        try:
            self.history.append(self.currentDirectory)
            try:
                self.currentDirectory = self.future.pop()
            except:
                pass
            self.goTo(self.currentDirectory, False)
            if len(self.future)==0:
                self.actForward.setEnabled(False)
        except:
            error = ReportBug.ReportBug()
            error.show()

    def goUp(self):
        try:
            self.goTo(trForM(InputOutputs.IA.getDirName(self.currentDirectory)))
        except:
            error = ReportBug.ReportBug()
            error.show()

    def goHome(self):
        try:
            self.goTo(MDir.homePath())
        except:
            error = ReportBug.ReportBug()
            error.show()

    def makeRefresh(self, _newDirectoryPath="", _isOnlyBrowser=False):
        try:
            if _newDirectoryPath!="" and _newDirectoryPath!=True and _newDirectoryPath!=False:
                self.goTo(_newDirectoryPath, False)
            else:
                if InputOutputs.IA.checkSource(str(self.currentDirectory), "directory")!=False:
                    self.makeRefreshOnlyFileList(self.lstvFileManager.rootIndex())
                    if _isOnlyBrowser==False:
                        self.showInTable()
                else:
                    self.goTo(InputOutputs.IA.getRealDirName(str(self.currentDirectory)), False)
        except:
            error = ReportBug.ReportBug()
            error.show()
    
    def makeRefreshOnlyFileList(self, _index=""):
        if _index=="":_index = self.lstvFileManager.currentIndex()
        if Universals.isActivePyKDE4==True:
            return self.dirModelMain.itemForIndex(self.dirModel.mapToSource(_index)).refresh()
        else:
            return self.dirModel.refresh(_index)
    
    def getPathOfIndex(self, _index):
        if Universals.isActivePyKDE4==True:
            return self.dirModelMain.itemForIndex(self.dirModel.mapToSource(_index)).url().pathOrUrl()
        else:
            return self.dirModel.filePath(_index)
        
    def getFileInfo(self, _index):
        if Universals.isActivePyKDE4==True:
            return self.dirModelMain.itemForIndex(self.dirModel.mapToSource(_index))
        else:
            return self.dirModel.fileInfo(_index)
    
    def leNavigatorPressed(self):
        self.goTo(str(self.currentDirectory + "/" + self.leNavigator.text()))
        self.leNavigator.setText("")
    
    def urlNavigatorUrlChanged(self, _murl):
        if self.isGoToFromUrlNavigator:
            self.goTo(_murl.pathOrUrl(), True)
        
    def dirOperatorUrlChanged(self, _murl):
        if self.isGoToFromDirOperator:
            self.goTo(_murl.pathOrUrl(), True)
        
    def setPlacesUrlChanged(self, _murl):
        self.goTo(_murl.pathOrUrl(), True)
        
    def setMyCurrentIndex(self, _index):
        try:
            while 1==1:
                selected = str(self.getPathOfIndex(_index))
                if InputOutputs.IA.isDir(selected)==True or InputOutputs.IA.isFile(selected)==True:
                    self.makeRefreshOnlyFileList(_index)
                    break
                else:
                    _index = _index.parent()
            self.goTo(self.getPathOfIndex(_index))
        except:
            error = ReportBug.ReportBug()
            error.show()
    
    def showInTable(self):
        try:
            Universals.MainWindow.Table.refresh(self.getCurrentDirectoryPath())
        except:
            error = ReportBug.ReportBug()
            error.show()
            
    def getCurrentDirectoryPath(self):
        return str(self.currentDirectory).replace("file://", "")
          
          
class BookmarksMenu(MMenu):
    def __init__(self, _parent):
        MMenu.__init__(self)
        self._parent = _parent;
        self.setTitle(translate("BookmarksMenu", "Bookmarks"))
        self.setIcon(MIcon("Images:bookmarks.png"))
        MObject.connect(self,SIGNAL("triggered(QAction *)"),self.triggered)
        self.makeRefresh()
    
    def makeRefresh(self):
        try:
            self.clear()
            for fav in Databases.BookmarksOfDirectories.fetchAll():
                self.addAction(trForUI(fav[1])).setObjectName(trForUI(fav[1]))
            self.addAction(translate("BookmarksMenu", "Edit Bookmarks")).setObjectName(translate("BookmarksMenu", "Edit Bookmarks"))
        except:
            error = ReportBug.ReportBug()
            error.show()

    def triggered(self, _action):
        try:
            if _action.objectName()==translate("BookmarksMenu", "Edit Bookmarks"):
                Universals.MainWindow.FileManager.bookmarks.makeRefresh()
                Universals.MainWindow.FileManager.bookmarks.show()
                return
            for info in Databases.BookmarksOfDirectories.fetchAll():
                if info[1]==str(_action.objectName()):
                    if InputOutputs.IA.isDir(str(info[2]))==True:
                        Universals.MainWindow.FileManager.goTo(trForM(info[2]))
                        return
                    else:
                        answer = Dialogs.ask(translate("BookmarksMenu", "Cannot Find Folder"), 
                                            str(translate("BookmarksMenu", "\"%s\" cannot be found.<br>Delete this folder from the bookmarks?")) % Organizer.getLink(info[1]))
                        if answer==Dialogs.Yes:
                            Databases.BookmarksOfDirectories.delete(str(info[0]))
                            self.makeRefresh()
                            Universals.MainWindow.FileManager.bookmarks.makeRefresh()
            Universals.MainWindow.FileManager.makeRefreshOnlyFileList()   
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    
class Bookmarks(MDialog):
    def __init__(self, _parent):
        MDialog.__init__(self)
        self._parent = _parent;
        if Universals.isActivePyKDE4==True:
            self.setButtons(MDialog.NoDefault)
        self.setWindowTitle(translate("Bookmarks", "Bookmarks"))
        pbtnDeleteBookmark = MPushButton(translate("Bookmarks", "Delete"))
        pbtnSaveBookmark = MPushButton(translate("Bookmarks", "Save"))
        pbtnClose = MPushButton(translate("Bookmarks", "Close"))
        MObject.connect(pbtnDeleteBookmark, SIGNAL("clicked()"), self.deleteBookmark)
        MObject.connect(pbtnSaveBookmark, SIGNAL("clicked()"), self.saveBookmark)
        MObject.connect(pbtnClose, SIGNAL("clicked()"), self.close)
        self.cbBookmarks = MComboBox()
        self.cbBookmarks.setEditable(True)
        self.pathOfBookmark = MLineEdit()
        pnlMain = MWidget(self)
        hbox = MHBoxLayout()
        hbox.addWidget(self.cbBookmarks)
        hbox.addWidget(pbtnDeleteBookmark)
        hbox1 = MHBoxLayout()
        hbox1.addWidget(self.pathOfBookmark)
        hbox1.addWidget(pbtnSaveBookmark)
        vblMain = MVBoxLayout(pnlMain)
        vblMain.addLayout(hbox)
        vblMain.addLayout(hbox1)
        vblMain.addWidget(pbtnClose)
        if Universals.isActivePyKDE4==True:
            self.setMainWidget(pnlMain)
        else:
            self.setLayout(vblMain)
        self.setFixedSize(300,120)
        MObject.connect(self.cbBookmarks, SIGNAL("currentIndexChanged(int)"), self.bookmarksChanged)
        self.makeRefresh()

    def closeEvent(self, _event):
        Universals.MainWindow.FileManager.bookmarksMenu.makeRefresh()
        
    def bookmarksChanged(self, _index):
        try:
            self.pathOfBookmark.setText(trForUI(Databases.BookmarksOfDirectories.fetchAll()[self.cbBookmarks.currentIndex()][2]))
        except:
            error = ReportBug.ReportBug()
            error.show()
            
    def addBookmark(self):
        try:
            currentPath = Universals.MainWindow.FileManager.getCurrentDirectoryPath()
            Databases.BookmarksOfDirectories.insert(currentPath.split("/")[-1], currentPath)
            Universals.MainWindow.FileManager.bookmarksMenu.makeRefresh()
            self.makeRefresh()
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def saveBookmark(self):
        try:
            info = Databases.BookmarksOfDirectories.fetchAll()[self.cbBookmarks.currentIndex()]
            Databases.BookmarksOfDirectories.update(info[0], str(self.cbBookmarks.currentText()), str(self.pathOfBookmark.text()))
            self.makeRefresh()
            Universals.MainWindow.FileManager.bookmarksMenu.makeRefresh()
        except:
            error = ReportBug.ReportBug()
            error.show()
            
    def deleteBookmark(self):
        try:
            if self.cbBookmarks.currentIndex()!=-1:
                info = Databases.BookmarksOfDirectories.fetchAll()[self.cbBookmarks.currentIndex()]
                Databases.BookmarksOfDirectories.delete(str(info[0]))
                self.makeRefresh()
                Universals.MainWindow.FileManager.bookmarksMenu.makeRefresh()
        except:
            error = ReportBug.ReportBug()
            error.show()
            
    def makeRefresh(self):
        try:
            self.cbBookmarks.clear()
            for fav in Databases.BookmarksOfDirectories.fetchAll():
                self.cbBookmarks.addItem(trForUI(fav[1])) 
        except:
            error = ReportBug.ReportBug()
            error.show()
            
            
            
            



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Variables
import Execute
from MyObjects import translate
pluginName = str(translate("MyPlugins/KDE4_CM", "For KDE4 Applications` Context Menus"))
pluginVersion = "0.4"
pluginFiles = ["HamsiManager_KDE4_CM.desktop", 
               "HamsiManager_KDE4_CM_Dir.desktop", 
               "HamsiManager_KDE4_CM_File.desktop"]
pluginDirectory = ""
installThisPlugin = None
if Variables.isRunningAsRoot():
    setupDirectory = "/usr/share/kde4/services/ServiceMenus"
else:
    setupDirectory = Variables.getKDE4HomePath() + "/share/kde4/services/ServiceMenus"

def isInstallable():
    return Variables.isAvailableKDE4()



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Organizer
import InputOutputs
import SearchEngines
from MyObjects import *
from Details import MusicDetails
import Universals
import Dialogs
import Taggers
from time import gmtime
import Records

class Content():
    global readContents, writeContents
    
    def readContents(_directoryPath):
        currentTableContentValues = []
        musicFileNames = InputOutputs.readDirectory(_directoryPath, "music")
        isCanNoncompatible = False
        allItemNumber = len(musicFileNames)
        Universals.startThreadAction()
        baseNameOfDirectory = InputOutputs.getBaseName(_directoryPath)
        for musicNo,musicName in enumerate(musicFileNames):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isReadableFileOrDir(_directoryPath+"/"+musicName):
                    tagger = Taggers.getTagger()
                    tagger.loadFile(_directoryPath+"/"+musicName)
                    if tagger.isAvailableFile() == False:
                        isCanNoncompatible=True
                    content = {}
                    content["path"] = _directoryPath + "/" + musicName
                    content["baseNameOfDirectory"] = baseNameOfDirectory
                    content["baseName"] = musicName
                    content["artist"] = tagger.getArtist()
                    content["title"] = tagger.getTitle()
                    content["album"] = tagger.getAlbum()
                    content["trackNum"] = tagger.getTrackNum()
                    content["year"] = tagger.getYear()
                    content["genre"] = tagger.getGenre()
                    content["firstComment"] = tagger.getFirstComment()
                    content["firstLyrics"] = tagger.getFirstLyrics()
                    currentTableContentValues.append(content)
            else:
                allItemNumber = musicNo+1
            Dialogs.showState(translate("InputOutputs/Musics", "Reading Music Tags"),musicNo+1,allItemNumber, True)
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        if isCanNoncompatible == True:
            Dialogs.show(translate("InputOutputs/Musics", "Possible ID3 Mismatch"),
                translate("InputOutputs/Musics", "Some of the files presented in the table may not support ID3 technology.<br>Please check the files and make sure they support ID3 information before proceeding."))
        return currentTableContentValues
    
    def writeContents(_table):
        _table.changedValueNumber = 0
        changingFileDirectories=[]
        Universals.startThreadAction()
        allItemNumber = len(_table.currentTableContentValues)
        Dialogs.showState(translate("InputOutputs/Musics", "Writing Music Tags"),0,allItemNumber, True)
        for rowNo in range(_table.rowCount()):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isWritableFileOrDir(_table.currentTableContentValues[rowNo]["path"]):
                    if _table.isRowHidden(rowNo):
                        InputOutputs.IA.removeFileOrDir(_table.currentTableContentValues[rowNo]["path"])
                        continue
                    baseNameOfDirectory = str(_table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                    baseName = str(_table.currentTableContentValues[rowNo]["baseName"])
                    tagger = Taggers.getTagger()
                    tagger.loadFileForWrite(_table.currentTableContentValues[rowNo]["path"])
                    if _table.isChangableItem(rowNo, 2, _table.currentTableContentValues[rowNo]["artist"]):
                        value = str(_table.item(rowNo,2).text())
                        tagger.setArtist(value)
                        Records.add(str(translate("MusicTable", "Artist")), str(_table.currentTableContentValues[rowNo]["artist"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 3, _table.currentTableContentValues[rowNo]["title"]):
                        value = str(_table.item(rowNo,3).text())
                        tagger.setTitle(value)
                        Records.add(str(translate("MusicTable", "Title")), str(_table.currentTableContentValues[rowNo]["title"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 4, _table.currentTableContentValues[rowNo]["album"]):
                        value = str(_table.item(rowNo,4).text())
                        tagger.setAlbum(value)
                        Records.add(str(translate("MusicTable", "Album")), str(_table.currentTableContentValues[rowNo]["album"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 5, _table.currentTableContentValues[rowNo]["trackNum"]):
                        value = str(_table.item(rowNo,5).text())
                        tagger.setTrackNum(value, len(_table.currentTableContentValues))
                        Records.add(str(translate("MusicTable", "Track No")), str(_table.currentTableContentValues[rowNo]["trackNum"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 6, _table.currentTableContentValues[rowNo]["year"]):
                        value = str(_table.item(rowNo,6).text())
                        tagger.setDate(value)
                        Records.add(str(translate("MusicTable", "Year")), str(_table.currentTableContentValues[rowNo]["year"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 7, _table.currentTableContentValues[rowNo]["genre"]):
                        value = str(_table.item(rowNo,7).text())
                        tagger.setGenre(value)
                        Records.add(str(translate("MusicTable", "Genre")), str(_table.currentTableContentValues[rowNo]["genre"]), value)
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 8, _table.currentTableContentValues[rowNo]["firstComment"]):
                        value = str(_table.item(rowNo,8).text())
                        tagger.setFirstComment(value)
                        Records.add(str(translate("MusicTable", "Comment")), str(_table.currentTableContentValues[rowNo]["firstComment"]), value)
                        _table.changedValueNumber += 1
                    if len(_table.tableColumns)>9 and _table.isChangableItem(rowNo, 9, _table.currentTableContentValues[rowNo]["firstLyrics"]):
                        value = str(_table.item(rowNo,9).text())
                        tagger.setFirstLyrics(value)
                        Records.add(str(translate("MusicTable", "Lyrics")), str(_table.currentTableContentValues[rowNo]["firstLyrics"]), value)
                        _table.changedValueNumber += 1
                    tagger.update()
                    if _table.isChangableItem(rowNo, 0, baseNameOfDirectory):
                        baseNameOfDirectory = str(_table.item(rowNo,0).text())
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 1, baseName, False):
                        baseName = str(_table.item(rowNo,1).text())
                        _table.changedValueNumber += 1
                    newFilePath = InputOutputs.getDirName(InputOutputs.getDirName(_table.currentTableContentValues[rowNo]["path"])) + "/" + baseNameOfDirectory + "/" + baseName
                    if InputOutputs.getRealPath(_table.currentTableContentValues[rowNo]["path"]) != InputOutputs.getRealPath(newFilePath):
                        changingFileDirectories.append([_table.currentTableContentValues[rowNo]["path"], 
                                                        newFilePath])
            else:
                allItemNumber = rowNo+1
            Dialogs.showState(translate("InputOutputs/Musics", "Writing Music Tags"),rowNo+1,allItemNumber, True)
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        InputOutputs.IA.changeDirectories(changingFileDirectories)
        return True



class MusicTable():
    def __init__(self, _table):
        self.Table = _table
        self.specialTollsBookmarkPointer = "music"
        self.hiddenTableColumnsSettingKey = "hiddenMusicTableColumns"
        self.refreshColumns()
        pbtnVerifyTableValues = MPushButton(translate("MusicTable", "Verify Table"))
        pbtnVerifyTableValues.setMenu(SearchEngines.SearchEngines(self.Table))
        self.Table.mContextMenu.addMenu(SearchEngines.SearchEngines(self.Table, True))
        self.isPlayNow = MToolButton()
        self.isPlayNow.setToolTip(translate("MusicTable", "Play Now"))
        self.isPlayNow.setIcon(MIcon("Images:playNow.png"))
        self.isPlayNow.setCheckable(True)
        self.isPlayNow.setAutoRaise(True)
        self.isPlayNow.setChecked(Universals.getBoolValue("isPlayNow"))
        self.Table.hblBox.insertWidget(self.Table.hblBox.count()-3, self.isPlayNow)
        self.Table.hblBox.insertWidget(self.Table.hblBox.count()-1, pbtnVerifyTableValues)
        
    def showDetails(self, _fileNo, _infoNo):
        MusicDetails.MusicDetails(self.Table.currentTableContentValues[_fileNo]["path"],
                                      self.Table.isOpenDetailsOnNewWindow.isChecked(), self.isPlayNow.isChecked())
    
    def cellClicked(self,_row,_column):
        cellLenght = len(self.Table.currentItem().text())*8
        if cellLenght > self.Table.columnWidth(_column):
            self.Table.setColumnWidth(_column,cellLenght)
        if _column==8 or _column==9:
            if self.Table.rowHeight(_row)<150:
                self.Table.setRowHeight(_row,150)
            if self.Table.columnWidth(_column)<250:
                self.Table.setColumnWidth(_column,250)
        
    def cellDoubleClicked(self,_row,_column):
        try:
            if _column==8 or _column==9:
                self.showDetails(_row, _column)
            else:
                if self.Table.tbIsRunOnDoubleClick.isChecked()==True:
                    self.showDetails(_row, _column)
        except:
            Dialogs.showError(translate("MusicTable", "Cannot Open Music File"), 
                        str(translate("MusicTable", "\"%s\" : cannot be opened. Please make sure that you selected a music file.")
                        ) % Organizer.getLink(self.Table.currentTableContentValues[_row]["path"]))
       
    def refreshColumns(self):
        self.Table.tableColumns = Taggers.getAvailableLabelsForTable()
        self.Table.tableColumnsKey = Taggers.getAvailableKeysForTable()
        
    def save(self):
        self.Table.checkFileExtensions(1, "baseName")
        MusicDetails.closeAllMusicDialogs()
        return writeContents(self.Table)
        
    def refresh(self, _path):
        self.Table.setColumnWidth(5,70)
        self.Table.setColumnWidth(6,40)
        self.Table.currentTableContentValues = readContents(_path)
        self.Table.setRowCount(len(self.Table.currentTableContentValues))
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(len(self.Table.tableColumns)):
                item = None
                if itemNo==0:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"], "directory")
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                elif itemNo==1:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseName"], "file")
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseName"])
                elif itemNo==2:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["artist"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["artist"])
                elif itemNo==3:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["title"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["title"])
                elif itemNo==4:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["album"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["album"])
                elif itemNo==5:
                    newString_temp = str(self.Table.currentTableContentValues[rowNo]["trackNum"]).split("/")
                    if newString_temp[0]=="None":
                        newString_temp[0]=str(rowNo+1)
                    newString = newString_temp[0]
                    newString += "/"+str(len(self.Table.currentTableContentValues))
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["trackNum"])
                elif itemNo==6:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["year"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["year"])
                elif itemNo==7:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["genre"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["genre"])
                elif itemNo==8:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["firstComment"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["firstComment"])
                elif itemNo==9:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["firstLyrics"])
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["firstLyrics"])
                if item!=None:
                    self.Table.setItem(rowNo, itemNo, item)
                        
    def correctTable(self):
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(self.Table.columnCount()):
                if itemNo==0:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), "directory")
                elif itemNo==1:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), "file")
                else:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()))
                self.Table.item(rowNo,itemNo).setText(trForUI(newString))
                



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from InputOutputs import Musics
import InputOutputs
import os,sys
from MyObjects import *
import Dialogs
import Organizer
from Viewers import MusicPlayer
from Details.ImageDetails import ImageDetails, closeAllImageDialogs
import Universals
import ReportBug
import Taggers
from Taggers import EyeD3Tagger

class MusicDetails(MDialog):
    global musicDialogs, closeAllMusicDialogs
    musicDialogs =[]
    def __init__(self, _filePath, _isOpenDetailsOnNewWindow=True, _isPlayNow=False):
        global musicDialogs
        if InputOutputs.IA.isFile(_filePath):
            if _isOpenDetailsOnNewWindow==False:
                isHasOpenedDialog=False
                for dialog in musicDialogs:
                    if dialog.isVisible()==True:
                        isHasOpenedDialog=True
                        self = dialog
                        self.changeFile(_filePath)
                        self.checkMusicTagType()
                        self.activateWindow()
                        self.raise_()
                        self.player.play(_filePath, _isPlayNow)
                        break
                if isHasOpenedDialog==False:
                    _isOpenDetailsOnNewWindow=True
            if _isOpenDetailsOnNewWindow==True:
                musicDialogs.append(self)
                MDialog.__init__(self, MApplication.activeWindow())
                if Universals.isActivePyKDE4==True:
                    self.setButtons(MDialog.NoDefault)
                self.isActiveAddImage = False
                self.infoLabels = {}
                self.infoValues = {}
                self.musicValues = {}
                self.pbtnClose = MPushButton(translate("MusicDetails", "Close"))
                self.pbtnSave = MPushButton(translate("MusicDetails", "Save Changes"))
                self.pbtnSave.setIcon(MIcon("Images:save.png"))
                MObject.connect(self.pbtnClose, SIGNAL("clicked()"), self.close)
                MObject.connect(self.pbtnSave, SIGNAL("clicked()"), self.save)
                self.labels = [translate("MusicDetails", "Directory: "),
                                translate("MusicDetails", "File Name: "),
                                translate("MusicDetails", "Artist: "),
                                translate("MusicDetails", "Title: "),
                                translate("MusicDetails", "Album: "),
                                translate("MusicDetails", "Track: "),
                                translate("MusicDetails", "Year: "),
                                translate("MusicDetails", "Genre: ")]
                self.pnlMain = MWidget()
                self.vblMain = MVBoxLayout(self.pnlMain)
                self.pnlClearable = None
                self.changeFile(_filePath, True)
                
                
                buttonHBOXs=MHBoxLayout()
                buttonHBOXs.addWidget(self.pbtnSave)
                buttonHBOXs.addWidget(self.pbtnClose)
                self.vblMain.addLayout(buttonHBOXs)
                if Universals.isActivePyKDE4==True:
                    self.setMainWidget(self.pnlMain)
                else:
                    self.setLayout(self.vblMain)
                self.checkMusicTagType()
                self.show()
                self.player.play(_filePath, _isPlayNow)
        else:
            Dialogs.showError(translate("MusicDetails", "File Does Not Exist"), 
                    str(translate("MusicDetails", "\"%s\" does not exist.<br>Table will be refreshed automatically!<br>Please retry.")
                        ) % Organizer.getLink(trForUI(_filePath)))
            from Universals import MainWindow
            MainWindow.FileManager.makeRefresh()
    
    def checkMusicTagType(self):
        if Taggers.getSelectedTaggerTypeName()=="ID3 V2":
            self.tabwTabs.setTabEnabled(1, True)
            self.tabwTabs.setTabEnabled(2, True)
        else:
            self.tabwTabs.setTabEnabled(1, False)
            self.tabwTabs.setTabEnabled(2, False)
    
    def changeFile(self, _filePath, _isNew=False):
        self.musicFile = _filePath
        self.musicValues = Musics.readMusicFile(self.musicFile)
        self.setWindowTitle(trForUI(InputOutputs.getBaseName(self.musicFile)))  
        if self.pnlClearable != None:
            Universals.clearAllChilds(self.pnlClearable, True)
        self.pnlClearable = MWidget()
        self.vblMain.insertWidget(0, self.pnlClearable, 20)
        vblClearable = MVBoxLayout(self.pnlClearable)
        self.player = MusicPlayer.MusicPlayer(self, "dialog", _filePath)     
        self.infoLabels["baseNameOfDirectory"] = MLabel(self.labels[0]) 
        self.infoLabels["baseName"] = MLabel(self.labels[1]) 
        self.infoLabels["artist"] = MLabel(self.labels[2]) 
        self.infoLabels["title"] = MLabel(self.labels[3]) 
        self.infoLabels["album"] = MLabel(self.labels[4]) 
        self.infoLabels["trackNum"] = MLabel(self.labels[5]) 
        self.infoLabels["year"] = MLabel(self.labels[6]) 
        self.infoLabels["genre"] = MLabel(self.labels[7]) 
        self.infoValues["baseNameOfDirectory"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["baseNameOfDirectory"], "directory", False)))
        self.infoValues["baseName"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["baseName"], "file")))
        self.infoValues["artist"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["artist"])))
        self.infoValues["title"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["title"])))
        self.infoValues["album"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["album"])))
        self.infoValues["trackNum"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["trackNum"])))
        self.infoValues["year"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["year"])))
        self.infoValues["genre"] = MLineEdit(trForUI(Organizer.emend(self.musicValues["genre"])))
        self.infoValues["firstComment"] = MPlainTextEdit(trForUI(Organizer.emend(self.musicValues["firstComment"])))
        self.infoValues["firstLyrics"] = MPlainTextEdit(trForUI(Organizer.emend(self.musicValues["firstLyrics"])))
        self.infoValues["firstComment"].setLineWrapMode(MPlainTextEdit.NoWrap)
        self.infoValues["firstLyrics"].setLineWrapMode(MPlainTextEdit.NoWrap)

        self.isOpenImageDetailsOnNewWindow = MCheckBox(translate("MusicDetails", "Show Images In New Window"))
        self.pbtnAddImage = MPushButton(translate("MusicDetails", "Append"))
        self.pbtnDeleteImage = MPushButton(translate("MusicDetails", "Delete"))
        self.pbtnSaveAsImage = MPushButton(translate("MusicDetails", "Save As ..."))
        self.pbtnCancelAddImage = MPushButton(translate("MusicDetails", "Cancel"))
        self.pbtnSelectImage = MPushButton(translate("MusicDetails", "Choose Image"))
        MObject.connect(self.pbtnAddImage, SIGNAL("clicked()"), self.addImage)
        MObject.connect(self.pbtnDeleteImage, SIGNAL("clicked()"), self.deleteImage)
        MObject.connect(self.pbtnSaveAsImage, SIGNAL("clicked()"), self.saveAsImage)
        MObject.connect(self.pbtnCancelAddImage, SIGNAL("clicked()"), self.cancelAddImage)
        MObject.connect(self.pbtnSelectImage, SIGNAL("clicked()"), self.selectImage)
        self.leImagePath = MLineEdit("")
        self.lblImagePath = MLabel(translate("MusicDetails", "Image Path: "))
        self.cbImageType = MComboBox()
        self.cbImageType.addItems(EyeD3Tagger.getImageTypes())
        self.lblImageType = MLabel(translate("MusicDetails", "Image Type: "))
        
        self.lstwImages = MListWidget()
        self.lstwImages.setGridSize(MSize(250,100))
        self.lstwImages.setIconSize(MSize(98,98))
        MObject.connect(self.lstwImages, SIGNAL("doubleClicked(QModelIndex)"),self.openImageDetails)
        self.lstwImages.clear()
        for image in self.musicValues["images"]:
            pixmImage = MPixmap()
            pixmImage.loadFromData(image[3])
            icnImage = QIcon(pixmImage)
            icnImage.actualSize(MSize(98,98))
            item = MListWidgetItem(icnImage,image[1]+"\n("+image[2]+")")
            item.setSizeHint(MSize(1,100))
            self.lstwImages.addItem(item)
        
        HBOXs = []
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["baseNameOfDirectory"])
        HBOXs[-1].addWidget(self.infoValues["baseNameOfDirectory"])
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["baseName"])
        HBOXs[-1].addWidget(self.infoValues["baseName"])
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["artist"])
        HBOXs[-1].addWidget(self.infoValues["artist"])
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["title"])
        HBOXs[-1].addWidget(self.infoValues["title"])
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["album"])
        HBOXs[-1].addWidget(self.infoValues["album"])
        HBOXs.append(MHBoxLayout())
        HBOXs[5].addWidget(self.infoLabels["trackNum"])
        HBOXs[5].addWidget(self.infoValues["trackNum"])
        HBOXs[5].addWidget(self.infoLabels["year"])
        HBOXs[5].addWidget(self.infoValues["year"])
        HBOXs[5].addWidget(self.infoLabels["genre"])
        HBOXs[5].addWidget(self.infoValues["genre"])
        vblInfos = MVBoxLayout()
        for hbox in HBOXs:
            vblInfos.addLayout(hbox)
        imageBoxs=[]
        imageBoxs.append(MHBoxLayout())
        imageBoxs[0].addWidget(self.leImagePath)
        imageBoxs[0].addWidget(self.pbtnSelectImage)
        imageBoxs.append(MHBoxLayout())
        imageBoxs[1].addWidget(self.lblImageType)
        imageBoxs[1].addWidget(self.cbImageType)
        imageBoxs.append(MHBoxLayout())
        imageBoxs[2].addWidget(self.pbtnAddImage)
        imageBoxs[2].addWidget(self.pbtnDeleteImage)
        imageBoxs[2].addWidget(self.pbtnSaveAsImage)
        imageBoxs[2].addWidget(self.pbtnCancelAddImage)
        vblImages = MVBoxLayout()
        vblImages.addWidget(self.lstwImages)
        vblImages.addWidget(self.isOpenImageDetailsOnNewWindow)
        vblImages.addWidget(self.lblImagePath)
        vblImages.addLayout(imageBoxs[0])
        vblImages.addLayout(imageBoxs[1])
        vblImages.addLayout(imageBoxs[2])
        vblComments = MVBoxLayout()
        vblComments.addWidget(self.infoValues["firstComment"])
        vblLyrics = MVBoxLayout()
        vblLyrics.addWidget(self.infoValues["firstLyrics"])
        self.tabwTabs = MTabWidget(self.pnlMain)
        self.pnlComments = MWidget(self.tabwTabs)
        self.pnlLyrics = MWidget(self.tabwTabs)
        self.pnlImages = MWidget(self.tabwTabs)
        self.pnlImages.setLayout(vblImages)
        self.pnlComments.setLayout(vblComments)
        self.pnlLyrics.setLayout(vblLyrics)
        self.tabwTabs.addTab(self.pnlComments, translate("MusicDetails", "Comments"))
        self.tabwTabs.addTab(self.pnlLyrics, translate("MusicDetails", "Lyrics"))
        self.tabwTabs.addTab(self.pnlImages, translate("MusicDetails", "Images"))
        vblClearable.addWidget(self.player)
        vblClearable.addLayout(vblInfos)
        vblClearable.addWidget(self.tabwTabs)
        self.pbtnSelectImage.hide()
        self.leImagePath.hide()
        self.lblImagePath.hide()
        self.lblImageType.hide()
        self.cbImageType.hide()
        self.pbtnCancelAddImage.hide()
        
    
    def closeEvent(self, _event):
        try:
            self.player.stop()
        except:
            pass
        closeAllImageDialogs()
    
    def closeAllMusicDialogs():
        for dialog in musicDialogs:
            try:
                if dialog.isVisible()==True:
                    dialog.player.stop()
                    dialog.close()
            except:
                continue
        
    def save(self):
        try:
            import Records
            Records.setTitle(translate("MusicDetails", "Music File"))
            closeAllImageDialogs()
            newMusicValues={}
            newMusicValues["baseNameOfDirectory"] = str(self.infoValues["baseNameOfDirectory"].text())
            newMusicValues["baseName"] = str(self.infoValues["baseName"].text())
            newMusicValues["artist"] = str(self.infoValues["artist"].text())
            newMusicValues["title"] = str(self.infoValues["title"].text())
            newMusicValues["album"] = str(self.infoValues["album"].text())
            newMusicValues["trackNum"] = str(self.infoValues["trackNum"].text())
            newMusicValues["year"] = str(self.infoValues["year"].text())
            newMusicValues["genre"] = str(self.infoValues["genre"].text())
            newMusicValues["firstComment"] = str(self.infoValues["firstComment"].toPlainText())
            newMusicValues["firstLyrics"] = str(self.infoValues["firstLyrics"].toPlainText())
            newPath = Musics.writeMusicFile(self.musicValues, newMusicValues)
            if newPath!=self.musicValues["path"]:
                self.changeFile(newPath)
            from Universals import MainWindow
            MainWindow.FileManager.makeRefresh()
            Records.saveAllRecords()
        except:
            error = ReportBug.ReportBug()
            error.show()  
        
    def openImageDetails(self, _index):
        try:
            ImageDetails(self.musicValues["images"][_index.row()][3], "data", self.isOpenImageDetailsOnNewWindow.isChecked())
        except:
            error = ReportBug.ReportBug()
            error.show()  
        
    def addImage(self):
        if self.isActiveAddImage==False:
            self.isActiveAddImage=True
            self.pbtnAddImage.setText(translate("MusicDetails", "OK"))
            self.pbtnSelectImage.show()
            self.leImagePath.show()
            self.lblImagePath.show()
            self.lblImageType.show()
            self.cbImageType.show()
            self.pbtnCancelAddImage.show()
            self.pbtnDeleteImage.hide()
            self.pbtnSaveAsImage.hide()
        else:
            if InputOutputs.IA.isFile(self.leImagePath.text())==True:
                closeAllImageDialogs()
                Musics.writeMusicFile(self.musicValues,False,True,self.cbImageType.currentIndex(),str(str(self.leImagePath.text())))
                self.changeFile(self.musicFile)
                self.cancelAddImage()
            else:
                Dialogs.showError(translate("MusicDetails", "Image Does Not Exist"),
                    str(translate("MusicDetails", "\"%s\" does not exist.")
                        ) % Organizer.getLink(trForUI(self.leImagePath.text())))
    
    def deleteImage(self):
        if self.lstwImages.currentRow()!=-1:
            closeAllImageDialogs()
            Musics.writeMusicFile(self.musicValues,False,True,self.musicValues["images"][self.lstwImages.currentRow()][0],False)
            self.changeFile(self.musicFile)
    
    def saveAsImage(self):
        try:
            if self.lstwImages.currentRow()!=-1:
                imagePath = MFileDialog.getSaveFileName(self,translate("MusicDetails", "Save As"),
                                    InputOutputs.getDirName(self.musicValues["path"]), trForUI(str(translate("MusicDetails", "Images (*.%s)")) %(str(self.musicValues["images"][self.lstwImages.currentRow()][2]).split("/")[1])))
                if imagePath!="":
                    sourceFile = os.getenv("TMP")+"/HamsiManager-image-file."+self.musicValues["images"][self.lstwImages.currentRow()][2].split("/")[1]
                    InputOutputs.IA.writeToBinaryFile(sourceFile, self.musicValues["images"][self.lstwImages.currentRow()][3])
                    InputOutputs.IA.moveOrChange(sourceFile, str(imagePath))
        except:
            error = ReportBug.ReportBug()
            error.show()  
            
    def cancelAddImage(self):
        self.isActiveAddImage=False
        self.pbtnAddImage.setText(translate("MusicDetails", "Append"))
        self.pbtnSelectImage.hide()
        self.leImagePath.hide()
        self.lblImagePath.hide()
        self.lblImageType.hide()
        self.cbImageType.hide()
        self.pbtnCancelAddImage.hide()
        self.pbtnDeleteImage.show()
        self.pbtnSaveAsImage.show()
        
    def selectImage(self):
        try:
            imagePath = MFileDialog.getOpenFileName(self,translate("MusicDetails", "Choose Image"),
                InputOutputs.getDirName(self.musicValues["path"]),trForUI(str(translate("MusicDetails", "Images")) + " " + Variables.imageExtStringOnlyPNGAndJPG))
            if imagePath!="":
                self.leImagePath.setText(imagePath)
        except:
            error = ReportBug.ReportBug()
            error.show()  
            



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Organizer
import InputOutputs
from MyObjects import *
from Details import TextDetails
import Dialogs
from time import gmtime
import Universals

class Content():
    global readContents, writeContents
    
    def readContents(_directoryPath):
        currentTableContentValues = []
        fileAndDirectoryNames = InputOutputs.IA.readDirectory(_directoryPath, "fileAndDirectory")
        allItemNumber = len(fileAndDirectoryNames)
        Universals.startThreadAction()
        baseNameOfDirectory = InputOutputs.getBaseName(_directoryPath)
        for dirNo,dirName in enumerate(fileAndDirectoryNames):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isReadableFileOrDir(_directoryPath + "/" + dirName):
                    content = {}
                    content["path"] = _directoryPath + "/" + dirName
                    content["baseNameOfDirectory"] = baseNameOfDirectory
                    content["baseName"] = dirName
                    currentTableContentValues.append(content)
            else:
                allItemNumber = dirNo+1
            Dialogs.showState(translate("InputOutputs/Folders", "Reading Directory Informations"),dirNo+1,allItemNumber, True) 
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        return currentTableContentValues
    
    def writeContents(_table):
        _table.changedValueNumber = 0
        changingFileDirectories=[]
        Universals.startThreadAction()
        allItemNumber = len(_table.currentTableContentValues)
        Dialogs.showState(translate("InputOutputs/Folders", "Writing Directory Informations"),0,allItemNumber, True)
        for rowNo in range(_table.rowCount()):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isWritableFileOrDir(_table.currentTableContentValues[rowNo]["path"]):
                    if _table.isRowHidden(rowNo):
                        InputOutputs.IA.removeFileOrDir(_table.currentTableContentValues[rowNo]["path"], True)
                        continue
                    baseNameOfDirectory = str(_table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                    baseName = str(_table.currentTableContentValues[rowNo]["baseName"])
                    if _table.isChangableItem(rowNo, 0, baseNameOfDirectory):
                        baseNameOfDirectory = str(_table.item(rowNo,0).text())
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 1, baseName, False):
                        baseName = str(_table.item(rowNo,1).text())
                        _table.changedValueNumber += 1
                    newFilePath = InputOutputs.getDirName(InputOutputs.getDirName(_table.currentTableContentValues[rowNo]["path"])) + "/" + baseNameOfDirectory + "/" + baseName
                    if InputOutputs.getRealPath(_table.currentTableContentValues[rowNo]["path"]) != InputOutputs.getRealPath(newFilePath):
                        changingFileDirectories.append([_table.currentTableContentValues[rowNo]["path"], 
                                                        newFilePath])
            else:
                allItemNumber = rowNo+1
            Dialogs.showState(translate("InputOutputs/Folders", "Writing Directory Informations"),rowNo+1,allItemNumber, True)
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        InputOutputs.IA.changeDirectories(changingFileDirectories)
        return True



class FolderTable():
    def __init__(self, _table):
        self.Table = _table
        self.specialTollsBookmarkPointer = "directory"
        self.hiddenTableColumnsSettingKey = "hiddenFolderTableColumns"
        self.refreshColumns()
        
    def showDetails(self, _fileNo, _infoNo):
        TextDetails.TextDetails(self.Table.currentTableContentValues[_fileNo]["path"],self.Table.isOpenDetailsOnNewWindow.isChecked())
    
    def cellClicked(self,_row,_column):
        cellLenght = len(self.Table.currentItem().text())*8
        if cellLenght>self.Table.columnWidth(_column):
            self.Table.setColumnWidth(_column,cellLenght)
        
    def cellDoubleClicked(self,_row,_column):
        try:
            if self.Table.tbIsRunOnDoubleClick.isChecked()==True:
                self.showDetails(_row, _column)
        except:
            Dialogs.showError(translate("FolderTable", "Cannot Open File"), 
                        str(translate("FolderTable", "\"%s\" : cannot be opened. Please make sure that you selected a text file.")
                        ) % Organizer.getLink(self.Table.currentTableContentValues[_row]["path"]))
       
    def refreshColumns(self):
        self.Table.tableColumns=[translate("FolderTable", "Directory"), 
                            translate("FolderTable", "File/Directory Name")]
        self.Table.tableColumnsKey=["Directory", "File/Directory Name"]
        
    def save(self):
        self.Table.checkFileExtensions(1, "baseName", True)
        return writeContents(self.Table)
    
    def refresh(self, _path):
        self.Table.currentTableContentValues = readContents(_path)
        self.Table.setRowCount(len(self.Table.currentTableContentValues))
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(2):
                item = None
                if itemNo==0:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"], "directory")
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                elif itemNo==1:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseName"], InputOutputs.getObjectType(self.Table.currentTableContentValues[rowNo]["path"]))
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseName"])
                if item!=None:
                    self.Table.setItem(rowNo, itemNo, item)
                    
    def correctTable(self):
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(self.Table.columnCount()):
                if itemNo==0:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), "directory")
                else:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), InputOutputs.IA.getObjectType(self.Table.currentTableContentValues[rowNo]["path"]))
                self.Table.item(rowNo,itemNo).setText(trForUI(newString))
          



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys,os
import time
import Variables
import Universals
import InputOutputs
import Dialogs
import ReportBug
from MyObjects import *
from datetime import timedelta, datetime

class UpdateControl(MDialog):
    global isMakeUpdateControl
    def __init__(self,_parent, _isNotInstall=False):
        MDialog.__init__(self, _parent)
        QtWebKit = getMyObject("QtWebKit")
        if Universals.isActivePyKDE4==True:
            self.setButtons(MDialog.NoDefault)
        if _isNotInstall==False:
            if self.isUpdatable()==False:
                _isNotInstall = True
        self.isNotInstall = _isNotInstall
        self.pnlMain = MWidget()
        self.vblMain = MVBoxLayout(self.pnlMain)
        self.isDownloading=False
        self.lblInfo = MLabel("")
        self.lblInfo.setVisible(False)
        self.lblInfo.setWordWrap(True)
        self.lblInfo.setOpenExternalLinks(True)
        self.pbtnCancel = MPushButton(translate("UpdateControl", "Cancel"))
        if self.isNotInstall==False:
            self.pbtnDownloadAndInstall = MPushButton(translate("UpdateControl", "Download and Install"))
        else:
            self.pbtnDownloadAndInstall = MPushButton(translate("UpdateControl", "Download"))
        self.pbtnDownloadAndInstall.setVisible(False)
        self.pbtnShowDetails = MPushButton(translate("UpdateControl", "Details"))
        self.pbtnShowDetails.setCheckable(True)
        self.pbtnShowDetails.setEnabled(False)
        self.wvWeb = QtWebKit.QWebView()
        self.prgbState = MProgressBar()
        self.prgbState.setRange(0,100)
        self.pbtnCheckForDeveloperVersion = MPushButton(translate("UpdateControl", "Check For Developer Version"))
        self.pbtnCheckForDeveloperVersion.setVisible(False)
        self.connect(self.pbtnCheckForDeveloperVersion,SIGNAL("clicked()"),self.checkForDeveloperVersion)
        self.connect(self.pbtnCancel,SIGNAL("clicked()"),self.close)
        self.connect(self.pbtnDownloadAndInstall,SIGNAL("clicked()"),self.downloadAndInstall)
        self.connect(self.pbtnShowDetails,SIGNAL("toggled(bool)"),self.showDetails)
        self.connect(self.wvWeb,SIGNAL("loadProgress(int)"),self.loading)
        self.connect(self.wvWeb,SIGNAL("loadFinished(bool)"),self.loadFinished)
        self.vblMain.addWidget(self.prgbState)
        self.vblMain.addWidget(self.lblInfo)
        self.pbtnRemindMeLater = MPushButton(MApplication.translate("UpdateControl", "Remind Me Later And Close"))
        self.cbRemindMeLater = MSpinBox()
        self.pbtnRemindMeLater.setVisible(False)
        self.cbRemindMeLater.setVisible(False)
        self.cbRemindMeLater.setRange(1, int(Universals.MySettings["updateInterval"]))
        self.cbRemindMeLater.setValue(1) 
        self.connect(self.pbtnRemindMeLater,SIGNAL("clicked()"),self.remindMeLaterAndClose)
        HBoxRemindMeLater = MHBoxLayout()
        HBoxRemindMeLater.addWidget(self.cbRemindMeLater)
        HBoxRemindMeLater.addWidget(self.pbtnRemindMeLater)
        self.vblMain.addLayout(HBoxRemindMeLater)
        hbox0 = MHBoxLayout()
        hbox0.addWidget(self.pbtnShowDetails,1)
        hbox0.addStretch(1)
        hbox0.addWidget(self.pbtnDownloadAndInstall,1)
        hbox0.addStretch(1)
        hbox0.addWidget(self.pbtnCancel,1)
        self.vblMain.addWidget(self.pbtnCheckForDeveloperVersion)
        self.vblMain.addLayout(hbox0)
        self.setWindowTitle(translate("UpdateControl", "Checking for the updates"))
        self.details = MLabel("")
        self.details.setWordWrap(True)
        self.details.setOpenExternalLinks(True)
        self.details.setMinimumHeight(220)
        self.vblMain.insertWidget(1,self.details)  
        self.details.setVisible(False)  
        self.pbtnDownloadAndInstall.setFixedWidth(180)
        self.setFixedWidth(400)
        self.setFixedHeight(130)   
        if Universals.isActivePyKDE4==True:
            self.setMainWidget(self.pnlMain)
        else:
            self.setLayout(self.vblMain)
        self.show()
        self.wvWeb.setUrl(MUrl("http://hamsiapps.com/ForMyProjects/UpdateControl.php?p=HamsiManager&v=" + str(Variables.intversion) + "&l=" + str(Universals.MySettings["language"])))
    
    def checkForDeveloperVersion(self):
        self.wvWeb.setUrl(MUrl("http://hamsiapps.com/ForMyProjects/UpdateControl.php?p=HamsiManager&v=" + str(Variables.intversion) + "&m=develop&l=" + str(Universals.MySettings["language"])))
    
    def remindMeLaterAndClose(self):
        Universals.setMySetting("remindMeLaterForUpdate", self.cbRemindMeLater.value())
        Universals.saveSettings()
        self.close()
    
    def loading(self, _value):
        self.prgbState.setValue(_value)
    
    def loadFinished(self, _bitti):
        try:
            if (_bitti):
                if self.isDownloading==False:
                    self.setFixedHeight(170)  
                    self.prgbState.setVisible(False)
                    self.pbtnShowDetails.setEnabled(True)
                    self.lblInfo.setVisible(True)
                    self.updateInformations=str(self.wvWeb.page().mainFrame().toPlainText()).split("\n")
                    if len(self.updateInformations)!=0:
                        if self.updateInformations[0][0]=="V":
                            self.pbtnRemindMeLater.setVisible(False)
                            self.cbRemindMeLater.setVisible(False)
                            self.pbtnDownloadAndInstall.setVisible(False)
                            self.pbtnCheckForDeveloperVersion.setVisible(False)
                            Universals.setMySetting("remindMeLaterForUpdate", "-1")
                            Universals.setMySetting("remindMeLaterShowDateForUpdate", datetime.now().strftime("%Y %m %d %H %M %S"))
                            Universals.saveSettings()
                            try:
                                lastVersion = int(self.updateInformations[0].replace("V", "").replace(".", ""))
                            except:
                                lastVersion = Variables.intversion -1
                            if lastVersion > Variables.intversion:
                                self.lblInfo.setText(str(translate("UpdateControl", "New release is available. Please download and install.<br>"+
                                                    "For details: <a href='%s' target='_blank'>Hamsi Manager</a>")) % (self.updateInformations[2]))
                                self.pbtnDownloadAndInstall.setVisible(True)
                                self.pbtnRemindMeLater.setVisible(True)
                                self.cbRemindMeLater.setVisible(True)
                                details = ""
                                for detail in self.updateInformations[4:]:
                                    details += detail+"<br>"
                                self.details.setText(str(translate("UpdateControl", "Version %s is available. Please download and install the new release.<br>"+
                                                      "%s For detailed information: <a href='%s' target='_blank'>Hamsi Manager</a><br>You can download from <a href='%s' target='_blank'>Hamsi Manager %s</a>")) % (self.updateInformations[0] + self.updateInformations[3], details, self.updateInformations[2], self.updateInformations[1], self.updateInformations[0]))
                            elif lastVersion < Variables.intversion:
                                self.lblInfo.setText(trForUI(str(translate("UpdateControl", "Lastest stable version is %s. You currently are using the version for developers.You can continue to use the current version.<br>For details: <a href='%s' target='_blank'>Hamsi Manager</a>")) % (self.updateInformations[0], self.updateInformations[2])))
                                self.pbtnDownloadAndInstall.setVisible(True)
                                if self.isNotInstall==False:
                                    self.pbtnDownloadAndInstall.setText(translate("UpdateControl", "Download and Install") + " (!)")
                                else:
                                    self.pbtnDownloadAndInstall.setText(translate("UpdateControl", "Download") + " (!)")
                                self.pbtnRemindMeLater.setVisible(True)
                                self.cbRemindMeLater.setVisible(True)
                                details = ""
                                for detail in self.updateInformations[4:]:
                                    details += detail+"<br>"
                                self.details.setText(trForUI(str(translate("UpdateControl", "Lastest stable version is %s. You currently are using the version for developers.You can continue to use the current version.<br>If you want a more accurate version, please download and install this version.<br>%s For detailed information: <a href='%s' target='_blank'>Hamsi Manager</a><br>You can download from <a href='%s' target='_blank'>Hamsi Manager %s</a>")) % (self.updateInformations[0] + self.updateInformations[3], details, self.updateInformations[2], self.updateInformations[1], self.updateInformations[0])))
                                self.pbtnCancel.setText(translate("UpdateControl", "Ok"))
                                self.pbtnCheckForDeveloperVersion.setVisible(True)
                            else:
                                self.details.setText(str(translate("UpdateControl", "For detailed information: <a href='%s' target='_blank'>Hamsi Manager</a>"))%(self.updateInformations[2])) 
                                self.lblInfo.setText(translate("UpdateControl", "You are already using the latest release."))
                                self.pbtnCancel.setText(translate("UpdateControl", "Ok"))
                            Universals.setMySetting("lastUpdateControlDate", datetime.now().strftime("%Y %m %d %H %M %S"))
                        else:
                            Universals.setMySetting("lastUpdateControlDate", datetime.now().strftime("%Y %m %d %H %M %S"))
                            Dialogs.showError(translate("UpdateControl", "Cannot Fetch Release Information"), 
                                        translate("UpdateControl", "Cannot fetch release information. Please retry later.<br>If you are constantly receiving this error, please visit \"http://hamsiapps.com/HamsiManager\"."))
                            self.close()
                    else:
                        Universals.setMySetting("lastUpdateControlDate", datetime.now().strftime("%Y %m %d %H %M %S"))
                        Dialogs.showError(translate("UpdateControl", "Cannot Fetch Release Information"), 
                                    translate("UpdateControl", "Cannot fetch release information. Please retry later.<br>If you are constantly receiving this error, please visit \"http://hamsiapps.com/HamsiManager\"."))
                        self.close()
                else:
                    self.lblInfo.setText(translate("UpdateControl", "Download complete."))
            else:
                Universals.setMySetting("lastUpdateControlDate", datetime.now().strftime("%Y %m %d %H %M %S"))
                Dialogs.showError(translate("UpdateControl", "Cannot Fetch Release Information"), 
                            translate("UpdateControl", "Cannot fetch release information. Please retry later.<br>If you are constantly receiving this error, please visit \"http://hamsiapps.com/HamsiManager\"."))
                self.close()
        except:
            error = ReportBug.ReportBug()
            error.show()  
            
    def showDetails(self, _value):
        if _value==True:
            self.details.setVisible(True) 
            self.setFixedHeight(330)    
        else:
            self.details.setVisible(False)  
            self.setFixedHeight(150)       
        
    def downloadAndInstall(self):
        try:
            if InputOutputs.isWritableFileOrDir(Variables.HamsiManagerDirectory):
                self.setFixedHeight(130)   
                self.isDownloading=True
                self.prgbState.setVisible(True)
                self.lblInfo.setVisible(False)
                self.setWindowTitle(translate("UpdateControl", "Downloading Latest Release..."))
                self.request = MNetworkRequest(MUrl(self.updateInformations[1]))
                self.willDownload(self.request)
            else:
                import Organizer
                Dialogs.showError(translate("UpdateControl", "Access Denied"),
                        str(translate("UpdateControl", "\"%s\" : you do not have the necessary permissions to change this directory.<br />Please check your access controls and retry. <br />Note: You can run Hamsi Manager as root and try again.")) % Organizer.getLink(realPath))
        except:
            error = ReportBug.ReportBug()
            error.show()  
          
    def willDownload(self, _request):
        try:
            defaultFileName = MFileInfo(trForM(Universals.trStr(_request.url()))).fileName()
            fileDialogTitle = translate("UpdateControl", "You Can Click Cancel To Update Without Saving The Package.")
            if self.isNotInstall:
                fileDialogTitle = translate("UpdateControl", "Save As")
            fileName = MFileDialog.getSaveFileName(self, fileDialogTitle,InputOutputs.IA.getDirName(Variables.HamsiManagerDirectory)+"/"+defaultFileName)
            if fileName== "":
                import random, tempfile
                fileName = tempfile.gettempdir() + "/" + defaultFileName[:-7]+"-"+str(random.randrange(0, 1000000))+defaultFileName[-7:]
            self.pbtnDownloadAndInstall.setEnabled(False)
            newRequest = _request
            newRequest.setAttribute(MNetworkRequest.User,Universals.trQVariant(fileName))
            networkManager = self.wvWeb.page().networkAccessManager()
            reply = networkManager.get(newRequest)
            self.isFileExist = True
            self.connect(reply,SIGNAL("downloadProgress(qint64,qint64)"),self.downloading)
            self.connect(reply,SIGNAL("finished()"),self.downloaded)
            self.connect(reply,SIGNAL("error(QNetworkReply::NetworkError)"),self.errorOccurred)
        except:
            error = ReportBug.ReportBug()
            error.show()  
        
    def downloading(self, _value, _maxValue):
        self.prgbState.setRange(0, _maxValue)
        self.prgbState.setValue(_value)
    
    def errorOccurred(self):
        self.isFileExist = False
        Dialogs.showError(translate("UpdateControl", "Cannot Read Source."), 
                    translate("UpdateControl", "Cannot read source package. Please retry later."))
        self.close()
        
    def downloaded(self):
        try:
            if self.isFileExist==True:
                self.prgbState.setVisible(False)
                self.pbtnDownloadAndInstall.setEnabled(False)
                self.pbtnCancel.setEnabled(False)
                reply = self.sender()
                request = reply.request()
                v = request.attribute(MNetworkRequest.User)
                fileName = Universals.trStr(v)
                InputOutputs.IA.writeToFile(fileName, reply.readAll())
                if self.isNotInstall==False:
                    self.setWindowTitle(translate("UpdateControl", "Installing The Latest Release"))
                    self.lblInfo.setText(translate("UpdateControl", "Latest release downloaded, initializing installation."))
                    self.install(fileName)
        except:
            error = ReportBug.ReportBug()
            error.show()  
        
    def install(self, _fileName):
        from Execute import executeWithPython
        Dialogs.show(translate("UpdateControl", "Update Will Be Complete"),
                        translate("UpdateControl", "Please restart Hamsi Manager now."),
                        translate("UpdateControl", "Restart"))
        if InputOutputs.IA.isFile(Variables.HamsiManagerDirectory+"/Update.py")==False:
            if InputOutputs.IA.isFile(Variables.HamsiManagerDirectory+"/ConfigureUpdate.py"):
                InputOutputs.IA.moveFileOrDir(Variables.HamsiManagerDirectory+"/ConfigureUpdate.py", Variables.HamsiManagerDirectory+"/Update.py")
        executeWithPython([Variables.HamsiManagerDirectory+"/Update.py", str(_fileName)])
        self.close()
        self.parent().close()
        
    def isUpdatable(self):
        return InputOutputs.IA.isWritableFileOrDir(Variables.HamsiManagerDirectory, True)
        
    def isMakeUpdateControl():
        lastUpdateControlTime = Universals.getDateValue("lastUpdateControlDate")
        updateInterval = int(Universals.MySettings["updateInterval"])
        if (lastUpdateControlTime + timedelta(days=updateInterval)).strftime("%Y%m%d%H%M%S") < datetime.now().strftime("%Y%m%d%H%M%S"):
            return True
        lastUpdateControlTime = Universals.getDateValue("remindMeLaterShowDateForUpdate")
        updateInterval = int(Universals.MySettings["remindMeLaterForUpdate"])
        if updateInterval!=-1:
            if (lastUpdateControlTime + timedelta(days=updateInterval)).strftime("%Y%m%d%H%M%S") < datetime.now().strftime("%Y%m%d%H%M%S"):
                return True
        return False
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import Universals
from Databases import sqlite, getDefaultConnection, correctForSql, getAmendedSQLInsertOrUpdateQueries

class CompleterTable:
    global fetchAll, fetchAllByObjectName, fetch, checkValues, insert, update, delete
    global getTableCreateQuery, getDeleteTableQuery, getDefaultsQueries
    global tableName, tableVersion, allForFetch, allForFetchByObjectName
    tableName = "completerTable"
    tableVersion = 1
    allForFetch, allForFetchByObjectName = None, {}
        
    def fetchAll():
        global allForFetch
        if allForFetch==None:
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute("SELECT * FROM " + tableName)
            allForFetch = cur.fetchall()
        return allForFetch
        
    def fetchAllByObjectName(_objectName=None):
        global allForFetchByObjectName
        if _objectName==None:
            _objectName = "%*%"
        if _objectName not in allForFetchByObjectName or allForFetchByObjectName[_objectName]==None:
            con = getDefaultConnection()
            cur = con.cursor()
            if _objectName=="%*%":
                cur.execute("SELECT DISTINCT value FROM " + tableName)
            else:
                cur.execute("SELECT DISTINCT value FROM " + tableName + " where objectName='" + _objectName + "' or objectName='*'")
            myValues = []
            for myval in cur.fetchall():
                myValues.append(myval[0])
            allForFetchByObjectName[_objectName] = myValues
        return allForFetchByObjectName[_objectName]
    
    def fetch(_id):
        con = getDefaultConnection()
        cur = con.cursor()
        cur.execute("SELECT * FROM " + tableName + " where id=" + str(int(_id)))
        return cur.fetchall()
    
    def checkValues(_objectName, _value):
        if len(_objectName)==0 or len(_value)==0:
            return False
        return True
    
    def insert(_objectName, _value):
        global allForFetch, allForFetchByObjectName
        if checkValues(_objectName, _value):
            allForFetch, allForFetchByObjectName[_objectName], allForFetchByObjectName["%*%"] = None, None, None
            con = getDefaultConnection()
            cur = con.cursor()
            sqlQueries = getAmendedSQLInsertOrUpdateQueries(tableName, {"objectName" : "'" + correctForSql(_objectName) + "'", "value" : "'" + correctForSql(_value) + "'"}, ["objectName", "value"])
            cur.execute(sqlQueries[0])
            cur.execute(sqlQueries[1])
            con.commit()
            cur.execute("SELECT last_insert_rowid();")
            return cur.fetchall()[0][0]
        return None
    
    def update(_id, _objectName, _value):
        global allForFetch, allForFetchByObjectName
        if checkValues(_objectName, _value):
            allForFetch, allForFetchByObjectName[_objectName], allForFetchByObjectName["%*%"] = None, None, None
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute(str("update " + tableName + " set objectName='" + correctForSql(_objectName) + "', value='" + correctForSql(_value) + "' where id=" + str(int(_id))))
            con.commit()
    
    def delete(_id):
        global allForFetch, allForFetchByObjectName
        allForFetch, allForFetchByObjectName = None, {}
        con = getDefaultConnection()
        cur = con.cursor()
        cur.execute("delete from " + tableName + " where id="+str(int(_id)))
        con.commit()
        
    def getTableCreateQuery():
        return "CREATE TABLE IF NOT EXISTS " + tableName + " ('id' INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,'objectName' TEXT,'value' TEXT)"
        
    def getDeleteTableQuery():
        return "DELETE FROM " + tableName
        
    def getDefaultsQueries():
        sqlQueries = []
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"objectName" : "'*'", "value" : "'Hamsi'"}, ["objectName", "value"])
        return sqlQueries
        
    
    
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import InputOutputs
from MyObjects import *
import Dialogs
import Organizer
import Universals
import ReportBug

MyDialog, MyDialogType, MyParent = getMyDialog()

class TextCorrector(MyDialog):
    
    def __init__(self,_filePath):
        MyDialog.__init__(self, MyParent)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setButtons(MyDialog.NoDefault)
        elif MyDialogType=="MMainWindow":
            self.setObjectName("Cleaner")
            Universals.MainWindow = self
        self.setWindowTitle(translate("TextCorrector", "Text Corrector")) 
        self.fileValues = None
        self.isChangeSourceCharSetChanged = False
        self.charSet = MComboBox()
        self.charSet.addItems(Variables.getCharSets())
        self.charSet.setCurrentIndex(self.charSet.findText(Universals.MySettings["fileSystemEncoding"]))
        self.sourceCharSet = MComboBox()
        self.sourceCharSet.addItems(Variables.getCharSets())
        self.sourceCharSet.setCurrentIndex(self.sourceCharSet.findText(Universals.MySettings["fileSystemEncoding"]))
        self.pbtnSelectFilePath = MPushButton(translate("TextCorrector", "Browse"))
        self.connect(self.pbtnSelectFilePath,SIGNAL("clicked()"), self.selectFilePath)
        pbtnClose = MPushButton(translate("TextCorrector", "Close"))
        self.pbtnSave = MPushButton(translate("TextCorrector", "Save Changes"))
        self.pbtnSave.setIcon(MIcon("Images:save.png"))
        MObject.connect(pbtnClose, SIGNAL("clicked()"), self.close)
        MObject.connect(self.pbtnSave, SIGNAL("clicked()"), self.save)
        self.labels = [translate("TextCorrector", "File Path : "), 
                            translate("TextCorrector", "Content : ")]
        pnlMain = MWidget(self)
        self.vblMain = MVBoxLayout(pnlMain)
        self.lblFilePath = MLabel(self.labels[0]) 
        self.lblFileContent = MLabel(self.labels[1])
        self.leFilePath = MLineEdit(trForUI(_filePath))
        self.pteFileContent = MPlainTextEdit(trForUI(""))
        self.pteFileContent.setLineWrapMode(MPlainTextEdit.NoWrap)
        self.fillValues()
        MObject.connect(self.sourceCharSet, SIGNAL("currentIndexChanged(int)"), self.sourceCharSetChanged)
        self.isChangeSourceCharSetChanged = True
        hbFilePath = MHBoxLayout()
        hbFilePath.addWidget(self.lblFilePath)
        hbFilePath.addWidget(self.leFilePath)
        hbFilePath.addWidget(self.pbtnSelectFilePath)
        hbFilePath.addWidget(self.sourceCharSet)
        self.vblMain.addLayout(hbFilePath)
        self.vblMain.addWidget(self.lblFileContent)
        self.vblMain.addWidget(self.pteFileContent)
        hbControls = MHBoxLayout()
        hbControls.addWidget(self.charSet, 1)
        hbControls.addWidget(self.pbtnSave, 4)
        hbControls.addWidget(pbtnClose, 1)
        self.vblMain.addLayout(hbControls, 1)
        self.leFilePath.setEnabled(False)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(self.vblMain)
        elif MyDialogType=="MMainWindow":
            self.setCentralWidget(pnlMain)
            moveToCenter(self)
        self.show()
        self.setMinimumWidth(700)
        self.setMinimumHeight(500)
                        
    def closeEvent(self, _event):
        MApplication.setQuitOnLastWindowClosed(True)
        
    def fillValues(self):
        filePath = str(self.leFilePath.text())
        if InputOutputs.isFile(filePath) and InputOutputs.IA.isReadableFileOrDir(filePath):
            
            self.fileValues = InputOutputs.readTextFile(filePath, str(self.sourceCharSet.currentText()))
            self.pteFileContent.setPlainText(trForUI(Organizer.emend(self.fileValues["content"], "text", False, True)))
            self.isChangeSourceCharSetChanged = True
            self.pbtnSave.setEnabled(True)
        else:
            self.isChangeSourceCharSetChanged = False
            self.pbtnSave.setEnabled(False)
            
    def sourceCharSetChanged(self):
        try:
            if self.isChangeSourceCharSetChanged:
                self.fillValues()
        except:
            self.pbtnSave.setEnabled(False)
            Dialogs.showError(translate("TextCorrector", "Incorrect File Encoding"), 
                        str(translate("TextCorrector", "File can not decode by \"%s\" codec.<br>Please select another file encoding type.")
                            )% trForUI(self.sourceCharSet.currentText()))

    def selectFilePath(self):
        try:
            filePath = MFileDialog.getOpenFileName(self,
                        translate("TextCorrector", "Please Select A Text File To Correct"), self.leFilePath.text(),
                        translate("TextCorrector", "All Files (*)"))
            if filePath!="":
                self.leFilePath.setText(filePath)
                self.isChangeSourceCharSetChanged = False
                self.sourceCharSet.setCurrentIndex(self.sourceCharSet.findText(Universals.MySettings["fileSystemEncoding"]))
                self.fillValues()
        except:
            error = ReportBug.ReportBug()
            error.show() 
        
    def save(self):
        try:
            filePath = str(self.leFilePath.text())
            if self.fileValues!=None:
                import Records
                Records.setTitle(translate("TextCorrector", "Text File"))
                newFileValues = {}
                newFileValues["path"] = filePath
                newFileValues["content"] = str(self.pteFileContent.toPlainText())
                newPath = InputOutputs.writeTextFile(self.fileValues, newFileValues, str(self.charSet.currentText()))
                if newPath!=self.fileValues["path"]:
                    self.changeFile(newPath)
                from Universals import MainWindow
                MainWindow.FileManager.makeRefresh()
                Records.saveAllRecords()
            else:
                Dialogs.showError(translate("TextCorrector", "File Does Not Exist"), 
                        str(translate("TextDetails", "\"%s\" does not exist.<br>Please select an exist file and try again.")
                            )% Organizer.getLink(trForUI(filePath)))
        except:
            error = ReportBug.ReportBug()
            error.show()  
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
from RoutineChecks import QuickMakeParameters
import RoutineChecks
import Dialogs, Records, Organizer
import InputOutputs
from MyObjects import *
import Universals
import ReportBug

class QuickMake():
    def __init__(self):
        if len(QuickMakeParameters)>1:
            answer = None
            isShowQuickMakeWindow = True
            tempWindow = MMainWindow()
            self.quickMakeWindow = QuickMakeWindow()
            Universals.MainWindow = self.quickMakeWindow
            isShowEmendWidgets = False
            isCorrectCommand = True
            if QuickMakeParameters[0]=="pack":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.pack
                actionName = translate("QuickMake", "Pack It Now")
            elif QuickMakeParameters[0]=="hash":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.hash
                actionName = translate("QuickMake", "Get Hash Digest")
            elif QuickMakeParameters[0]=="checkIcon":
                makeThisAction = self.quickMakeWindow.checkIcon
                actionName = translate("QuickMake", "Check Directory Icon Now")
            elif QuickMakeParameters[0]=="clearEmptyDirectories":
                makeThisAction = self.quickMakeWindow.clearEmptyDirectories
                actionName = translate("QuickMake", "Clear Empty Directories Now")
            elif QuickMakeParameters[0]=="clearUnneededs":
                makeThisAction = self.quickMakeWindow.clearUnneededs
                actionName = translate("QuickMake", "Clear Unneededs Now")
            elif QuickMakeParameters[0]=="clearIgnoreds":
                makeThisAction = self.quickMakeWindow.clearIgnoreds
                actionName = translate("QuickMake", "Clear Ignoreds Now")
            elif QuickMakeParameters[0]=="emendFile":
                isShowEmendWidgets = True
                makeThisAction = self.quickMakeWindow.emendFile
                actionName = translate("QuickMake", "Auto Emend Now")
            elif QuickMakeParameters[0]=="emendDirectory":
                isShowEmendWidgets = True
                makeThisAction = self.quickMakeWindow.emendDirectory
                actionName = translate("QuickMake", "Auto Emend Now")
            elif QuickMakeParameters[0]=="emendDirectoryWithContents":
                isShowEmendWidgets = True
                makeThisAction = self.quickMakeWindow.emendDirectoryWithContents
                actionName = translate("QuickMake", "Auto Emend Now (With Contents)")
            elif QuickMakeParameters[0]=="copyPath":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.copyPath
                actionName = translate("QuickMake", "Copy Path To Clipboard")
            elif QuickMakeParameters[0]=="fileTree":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.fileTree
                actionName = translate("QuickMake", "Build File Tree")
            elif QuickMakeParameters[0]=="removeOnlySubFiles":
                makeThisAction = self.quickMakeWindow.removeOnlySubFiles
                actionName = translate("QuickMake", "Remove Sub Files")
            elif QuickMakeParameters[0]=="clear":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.clear
                actionName = translate("QuickMake", "Clear It Now")
            elif QuickMakeParameters[0]=="textCorrector":
                isShowQuickMakeWindow = False
                makeThisAction = self.quickMakeWindow.textCorrector
                actionName = translate("QuickMake", "Text Corrector")
            else:
                isCorrectCommand = False
            if isCorrectCommand:
                if isShowQuickMakeWindow:
                    if Universals.getBoolValue("isShowQuickMakeWindow"):
                        self.quickMakeWindow.createWindow(actionName, makeThisAction, isShowEmendWidgets)
                        self.quickMakeWindow.show()
                    else:
                        makeThisAction()
                else:
                    makeThisAction()
            else:
                answer = Dialogs.askSpecial(translate("QuickMake", "Incorrect Command"),
                        translate("QuickMake", "Your action unable to process.Please try again or report this bug."), translate("QuickMake", "Report This Bug"), translate("QuickMake", "Close"))
        else:
            answer = Dialogs.askSpecial(translate("QuickMake", "Missing Command"),
                        translate("QuickMake", "Your action unable to process.Please try again or report this bug."), translate("QuickMake", "Report This Bug"), translate("QuickMake", "Close"))
        if answer==translate("QuickMake", "Report This Bug"):
            self.error = ReportBug.ReportBug(True)
            self.error.show()

MyDialog, MyDialogType, MyParent = getMyDialog()
   
class QuickMakeWindow(MyDialog):
    def __init__(self):
        MyDialog.__init__(self, MyParent)
        
    def createWindow(self, _actionName, _makeThisAction, _isShowEmendWidgets):
        import Options
        newOrChangedKeys = Universals.newSettingsKeys + Universals.changedDefaultValuesKeys
        wOptionsPanel = Options.Options(None, QuickMakeParameters[0], None, newOrChangedKeys)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setButtons(MyDialog.NoDefault)
        elif MyDialogType=="MMainWindow":
            self.setObjectName("Packager")
            Universals.MainWindow = self
        self.setWindowTitle(_actionName)
        pnlMain = MWidget(self)
        vblMain = MVBoxLayout(pnlMain)
        pnlInfo = MWidget()
        vblInfo = MVBoxLayout(pnlInfo)
        vblInfo.addStretch(3)
        if _isShowEmendWidgets:
            lblOldValue = MLabel(translate("QuickMake", "Old Value : "))
            lblNewValue = MLabel(translate("QuickMake", "New Value : "))
            leOldValue = MLineEdit(trForUI(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            leOldValue.setEnabled(False)
            self.leNewValue = MLineEdit(trForUI(Organizer.emend(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), InputOutputs.IA.getObjectType(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))))
            vblInfo.addWidget(lblOldValue)
            vblInfo.addWidget(leOldValue)
            vblInfo.addWidget(lblNewValue)
            vblInfo.addWidget(self.leNewValue)
        else:
            lblValue = MLabel(translate("QuickMake", "Value : "))
            leValue = MLineEdit(trForUI(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            leValue.setEnabled(False)
            vblInfo.addWidget(lblValue)
            vblInfo.addWidget(leValue)
        vblInfo.addStretch(3)
        pbtnApply = MPushButton(_actionName)
        pbtnClose = MPushButton(translate("QuickMake", "Cancel"))
        MObject.connect(pbtnApply, SIGNAL("clicked()"), _makeThisAction)
        MObject.connect(pbtnClose, SIGNAL("clicked()"), self.close)
        tabwTabs = MTabWidget()
        tabwTabs.addTab(pnlInfo, translate("QuickMake", "Quick Make"))
        tabwTabs.addTab(wOptionsPanel, translate("QuickMake", "Quick Make Options"))
        vblMain.addWidget(tabwTabs)
        hblBox = MHBoxLayout()
        hblBox.addWidget(pbtnClose, 2)
        hblBox.addWidget(pbtnApply, 3)
        vblInfo.addLayout(hblBox)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(vblMain)
        elif MyDialogType=="MMainWindow":
            self.setCentralWidget(pnlMain)
            moveToCenter(self)
        self.setMinimumSize(450, 350)
                        
    def closeEvent(self, _event):
        MApplication.setQuitOnLastWindowClosed(True)
    
    def checkSource(self, _oldPath, _objectType="fileOrDirectory"):
        if _objectType=="file" and InputOutputs.IA.isFile(_oldPath)==False:
            answer = Dialogs.ask(translate("QuickMake", "Cannot Find File"),
                    str(translate("InputOutputs", "\"%s\" : cannot find a file with this name.<br>Are you want to organize parant directory with Hamsi Manager?")) % Organizer.getLink(_oldPath))
            if answer==Dialogs.Yes:
                self.organizeWithHamsiManager(_oldPath)
            return False
        elif _objectType=="directory" and InputOutputs.IA.isDir(_oldPath)==False:
            answer = Dialogs.ask(translate("QuickMake", "Cannot Find Directory"),
                    str(translate("InputOutputs", "\"%s\" : cannot find a folder with this name.<br>Are you want to organize parant directory with Hamsi Manager?")) % Organizer.getLink(_oldPath))
            if answer==Dialogs.Yes:
                self.organizeWithHamsiManager(_oldPath)
            return False
        elif InputOutputs.IA.isDir(_oldPath)==False and InputOutputs.IA.isFile(_oldPath)==False:
            answer = Dialogs.ask(translate("QuickMake", "Cannot Find File Or Directory"),
                    str(translate("InputOutputs", "\"%s\" : cannot find a file or directory with this name.<br>Are you want to organize parant directory with Hamsi Manager?")) % Organizer.getLink(_oldPath))
            if answer==Dialogs.Yes:
                self.organizeWithHamsiManager(_oldPath)
            return False
        return True
        
    def organizeWithHamsiManager(self, _oldPath):
        Universals.setMySetting("lastDirectory", InputOutputs.IA.getRealDirName(_oldPath, True))
        RoutineChecks.isQuickMake = False
        self.close()
    
    def pack(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                from Tools import Packager
                self.newDialog = Packager.Packager(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
    
    def hash(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "file"):
                from Tools import Hasher
                self.newDialog = Hasher.Hasher(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                
    def checkIcon(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                InputOutputs.IA.checkIcon(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
                Dialogs.show(translate("QuickMake", "Directory Icon Checked"),
                        str(translate("QuickMake", "\"%s\"`s icon checked.<br>The default action based on the data is executed.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   

    def clearEmptyDirectories(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                InputOutputs.IA.activateSmartCheckIcon()
                InputOutputs.IA.clearEmptyDirectories(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), True, True)
                if InputOutputs.IA.isDir(InputOutputs.IA.getRealPath(QuickMakeParameters[1])):
                    InputOutputs.IA.completeSmartCheckIcon()
                Dialogs.show(translate("QuickMake", "Directory Cleaned"),
                        str(translate("QuickMake", "\"%s\" is cleaned based on the criteria you set.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                
    def clearUnneededs(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                InputOutputs.IA.clearUnneededs(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
                Dialogs.show(translate("QuickMake", "Directory Cleaned"),
                        str(translate("QuickMake", "\"%s\" is cleaned based on the criteria you set.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                
    def clearIgnoreds(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                InputOutputs.IA.clearIgnoreds(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
                Dialogs.show(translate("QuickMake", "Directory Cleaned"),
                        str(translate("QuickMake", "\"%s\" is cleaned based on the criteria you set.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                        
    def emendFile(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "file"):
                if Universals.getBoolValue("isShowQuickMakeWindow"):
                    newEmendedName = str(self.leNewValue.text())
                else:
                    newEmendedName = Organizer.emend(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), InputOutputs.IA.getObjectType(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
                import Organizer
                oldFileName = InputOutputs.IA.getRealPath(QuickMakeParameters[1])
                newFileName = InputOutputs.IA.moveOrChange(oldFileName, newEmendedName)
                if newFileName!=oldFileName:
                    Dialogs.show(translate("QuickMake", "File Emended"),
                            str(translate("QuickMake", "\"%s\" is emended based on the criteria you set.This file is \"%s\" now.")) % 
                            (Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])), Organizer.getLink(newFileName)))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   

    def emendDirectory(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                if Universals.getBoolValue("isShowQuickMakeWindow"):
                    newEmendedName = str(self.leNewValue.text())
                else:
                    newEmendedName = Organizer.emend(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), InputOutputs.IA.getObjectType(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
                import Organizer
                oldFileName = InputOutputs.IA.getRealPath(QuickMakeParameters[1])
                newDirName = InputOutputs.IA.moveOrChange(oldFileName, newEmendedName, "directory")
                if newDirName!=oldFileName:
                    Dialogs.show(translate("QuickMake", "Directory Emended"),
                            str(translate("QuickMake", "\"%s\" is emended based on the criteria you set.This directory is \"%s\" now.")) % 
                            (Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])), Organizer.getLink(newDirName)))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                            
    def emendDirectoryWithContents(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                if Universals.getBoolValue("isShowQuickMakeWindow"):
                    newEmendedName = str(self.leNewValue.text())
                else:
                    newEmendedName = Organizer.emend(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), InputOutputs.IA.getObjectType(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
                import Organizer
                InputOutputs.IA.activateSmartCheckIcon()
                oldFileName = InputOutputs.IA.getRealPath(QuickMakeParameters[1])
                newDirName = InputOutputs.IA.moveOrChange(oldFileName, newEmendedName, "directory")
                if newDirName!=oldFileName:
                    fileAndDirectoryNames = InputOutputs.IA.readDirectory(newDirName, "fileAndDirectory")
                    for fileAndDirs in fileAndDirectoryNames:
                        objectType = InputOutputs.IA.getObjectType(newDirName + "/" + fileAndDirs)
                        InputOutputs.IA.moveOrChange(newDirName + "/" + fileAndDirs, 
                                  newDirName + "/" + Organizer.emend(fileAndDirs, objectType), objectType)
                    if Universals.getBoolValue("isActiveAutoMakeIconToDirectory") and Universals.getBoolValue("isAutoMakeIconToDirectoryWhenFileMove"):
                        InputOutputs.IA.checkIcon(newDirName)
                    if InputOutputs.IA.isDir(newDirName):
                        InputOutputs.IA.completeSmartCheckIcon()
                    Dialogs.show(translate("QuickMake", "Directory And Contents Emended"),
                            str(translate("QuickMake", "\"%s\" is emended based on the criteria you set.This directory is \"%s\" now.")) % 
                            (Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])), Organizer.getLink(newDirName)))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   
                            
    def copyPath(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1])):
                MApplication.clipboard().setText(trForUI(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
                Dialogs.show(translate("QuickMake", "Copied To Clipboard"),
                        str(translate("QuickMake", "\"%s\" copied to clipboard.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   

    def fileTree(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                from Tools import FileTreeBuilder
                self.newDialog = FileTreeBuilder.FileTreeBuilder(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()     
    
    def removeOnlySubFiles(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                answer = Dialogs.ask(translate("QuickMake", "All Files Will Be Removed"),
                        str(translate("QuickMake", "Are you sure you want to remove only all files in \"%s\"?<br>Note:Do not will remove directory and subfolders.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
                if answer==Dialogs.Yes:
                    Universals.MainWindow.setEnabled(False)
                    InputOutputs.IA.removeOnlySubFiles(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
                    Universals.MainWindow.setEnabled(True)
                    Dialogs.show(translate("QuickMake", "Removed Only All Files"),
                        str(translate("QuickMake", "Removed only all files in \"%s\".<br>Note:Do not removed directory and subfolders.")) % Organizer.getLink(InputOutputs.IA.getRealPath(QuickMakeParameters[1])))
            self.close()
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()   

    def clear(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "directory"):
                from Tools import Cleaner
                self.newDialog = Cleaner.Cleaner(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()     
    
    def textCorrector(self):
        try:
            if self.checkSource(InputOutputs.IA.getRealPath(QuickMakeParameters[1]), "file"):
                from Tools import TextCorrector
                self.newDialog = TextCorrector.TextCorrector(InputOutputs.IA.getRealPath(QuickMakeParameters[1]))
        except:
            self.error = ReportBug.ReportBug()
            self.error.show()    
            
            
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os, sys
import subprocess
from threading import Thread
import time
import Variables
import InputOutputs

class Execute:
    global execute, executeAsThread, executeWithPython, writeToPopen, executeAsRoot, executeWithPythonAsRoot, executeHamsiManagerAsRoot, executeHamsiManager, executeReconfigure, executeReconfigureAsRoot, open, getCommandResult, executeStringCommand
        
    def getCommandResult(_command):
        if os.name=="nt":
            _command = ["start"] + _command
        myPopen = subprocess.Popen(_command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=True)
        po, pi = myPopen.stdin, myPopen.stdout
        po.close()
        return pi.read()
        
    def executeStringCommand(_command):
        if os.name=="nt":
            _command = "start" + _command
        return os.popen(_command)
        
    def execute(_command):
        return subprocess.Popen(_command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, bufsize=1)
    
    def executeAsThread(_command):
        roar = RunAsThread(_command)
        roar.start()
        time.sleep(1)
        return True
    
    def open(_command):
        if os.name=="nt":
            return os.startfile(_command[0])
        else:
            _command = ["xdg-open"] + _command
            return subprocess.Popen(_command)
        
    def executeWithPython(_command):
        return execute([sys.executable] + _command)
        
    def executeHamsiManager(_command=[]):
        return execute([sys.executable, str(Variables.executableHamsiManagerPath)] + _command )
        
    def executeReconfigure(_command=[]):
        return execute([sys.executable, str(Variables.HamsiManagerDirectory+"/Reconfigure.py")] + _command)
        
    def executeAsRoot(_command):
        if Variables.isRunableAsRoot():
            stringCommand = ""
            for myCommand in _command:
                if myCommand[:2]=="--":
                    stringCommand += "%s " % myCommand
                else:
                    stringCommand += "%r " % myCommand
            return execute([Variables.getLibraryDirectoryPath() + "/kde4/libexec/kdesu" , stringCommand ])
        return False
    
    def executeWithPythonAsRoot(_command):
        if Variables.isRunableAsRoot():
            return executeAsRoot([sys.executable] + _command)
        return False
        
    def executeHamsiManagerAsRoot(_command=[]):
        if Variables.isRunableAsRoot():
            roar = RunHamsiManagerAsRoot(_command)
            roar.start()
            time.sleep(1)
            return True
        return False
        
    def executeReconfigureAsRoot(_command=[]):
        if Variables.isRunableAsRoot():
            roar = RunReconfigureAsRoot(_command)
            roar.start()
            time.sleep(1)
            return True
        return False
        
    def writeToPopen(_popen, _command):
        _popen.stdin.write("\n%s\n" % _command)
        
class RunHamsiManagerAsRoot(Thread):
    def __init__(self, _command):
        Thread.__init__(self)
        self.command = _command
    
    def run(self):
        executeWithPythonAsRoot([Variables.executableHamsiManagerPath] + self.command)
        
class RunReconfigureAsRoot(Thread):
    def __init__(self, _command):
        Thread.__init__(self)
        self.command = _command
    
    def run(self):
        executeWithPythonAsRoot([Variables.HamsiManagerDirectory + "/Reconfigure.py"] + self.command)
        
class RunAsThread(Thread):
    def __init__(self, _command):
        Thread.__init__(self)
        self.command = _command
    
    def run(self):
        execute(self.command)
        
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
import os
import Variables
import InputOutputs
import Universals
import Settings

class MyConfigure:
    global reConfigureFile, installKDE4Language, installKDE4Languages, getDesktopFileContent, getConfiguredDesktopFileContent
    
    def reConfigureFile(_filePath, _installationDirectory=Variables.HamsiManagerDirectory):
        fileContent = InputOutputs.readFromFile(_filePath).replace("~InstallationDirectory~", _installationDirectory).replace("~SettingsDirectory~", Universals.pathOfSettingsDirectory).replace("~KDE4HomeDirectory~", Variables.getKDE4HomePath())
        InputOutputs.writeToFile(_filePath, fileContent)
            
    def installKDE4Languages():
        if Variables.isAvailableKDE4():
            for langCode in Variables.getInstalledLanguagesCodes():
                installKDE4Language(langCode)
            Universals.setMySetting("isInstalledKDE4Language", True)
            return True
        return False
            
    def installKDE4Language(_language="tr_TR", _KDELocalateDir = None):
        if Variables.isAvailableKDE4():
            if _KDELocalateDir==None:
                _KDELocalateDir = Variables.getKDE4HomePath() +"/share/locale/~langCode~/LC_MESSAGES/"
            import Execute
            if Variables.isRunningAsRoot():
                _KDELocalateDir = "/usr/share/locale/~langCode~/LC_MESSAGES/"
            _KDELocalateDir = str(_KDELocalateDir)
            _KDELocalateDir = _KDELocalateDir.replace("~langCode~", str(_language[:2]))
            langFile = Variables.HamsiManagerDirectory+"/Languages/" + str(_language)+".mo"
            if InputOutputs.isFile(_KDELocalateDir+"HamsiManager.mo")==False:
                if InputOutputs.isFile(langFile):
                    if InputOutputs.isDir(_KDELocalateDir)==False:
                        InputOutputs.makeDirs(_KDELocalateDir)
                    InputOutputs.copyFileOrDir(langFile,_KDELocalateDir+"HamsiManager.mo")
            return True
        return False
        
    def getDesktopFileContent():
        return ("#!/usr/bin/env xdg-open\n" +
            "[Desktop Entry]\n" +
            "Encoding=UTF-8\n" +
            "Comment[tr]=Hamsi Manager\n" +
            "Comment=Hamsi Manager\n" +
            "Categories=Audio;AudioVideo;AudioVideoEditing;X-MandrivaLinux-Multimedia-Sound;Qt;KDE;Utility;X-KDE-Utilities-File\n" +
            "Exec=python '~InstallationDirectory~/HamsiManager.py'\n" +
            "GenericName[tr]=Hamsi Manager\n" +
            "GenericName=Hamsi Manager\n" +
            "Icon=~InstallationDirectory~/Themes/Default/Images/HamsiManager-128x128.png\n" +
            "MimeType=\n" +
            "Name[tr]=Hamsi Manager\n" +
            "Name=Hamsi Manager\n" +
            "Path=~InstallationDirectory~\n" +
            "StartupNotify=true\n" +
            "Terminal=false\n" +
            "TerminalOptions=\n" +
            "Type=Application\n" +
            "X-DBUS-ServiceName=\n" +
            "X-DBUS-StartupType=\n" +
            "X-KDE-SubstituteUID=false\n" +
            "X-KDE-Username=\n")
        
    def getConfiguredDesktopFileContent(_installationDirectory=Variables.HamsiManagerDirectory):
        return getDesktopFileContent().replace("~InstallationDirectory~", _installationDirectory).replace("~SettingsDirectory~", Universals.pathOfSettingsDirectory).replace("~KDE4HomeDirectory~", Variables.getKDE4HomePath())

        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Organizer
import InputOutputs
from MyObjects import *
from Details import CoverDetails
import Dialogs
from time import gmtime
import Universals

class Content():
    global readContents, writeContents
    
    def readContents(_directoryPath):
        currentTableContentValues = []
        allFilesAndDirectories = InputOutputs.IA.readDirectoryWithSubDirectories(_directoryPath, 
                    int(Universals.MySettings["CoversSubDirectoryDeep"]), True, True)
        allItemNumber = len(allFilesAndDirectories)
        Universals.startThreadAction()
        for dirNo,dirName in enumerate(allFilesAndDirectories):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isReadableFileOrDir(dirName):
                    content = {}
                    content["path"] = dirName
                    content["baseNameOfDirectory"] = str(str(InputOutputs.IA.getBaseName(_directoryPath)) + 
                                    str(InputOutputs.IA.getDirName(dirName)).replace(_directoryPath,""))
                    content["baseName"] = InputOutputs.IA.getBaseName(dirName)

                    currentCover, isCorrectedFileContent = InputOutputs.IA.getIconFromDirectory(dirName)
                    selectedName = None
                    if isCorrectedFileContent and currentCover!=None:
                        selectedName = InputOutputs.IA.getBaseName(currentCover)
                    sourceCover = InputOutputs.IA.getFirstImageInDirectory(dirName, selectedName, False, False)
                    if currentCover==None:
                        currentCover = ""
                    if sourceCover==None:
                        sourceCover = ""
                    else:
                        sourceCover = dirName + "/" + sourceCover
                    content["currentCover"] = (currentCover)
                    content["sourceCover"] = (sourceCover)
                    content["destinationCover"] = (sourceCover)
                    content["isCorrectedFileContent"] = (isCorrectedFileContent)
                    currentTableContentValues.append(content)
            else:
                allItemNumber = dirNo+1
            Dialogs.showState(translate("InputOutputs/Covers", "Reading Cover Informations"),
                              dirNo+1,allItemNumber, True) 
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        return currentTableContentValues
    
    def writeContents(_table):
        _table.changedValueNumber = 0
        changingFileDirectories=[]
        startRowNo,rowStep=0,1
        Universals.startThreadAction()
        allItemNumber = len(_table.currentTableContentValues)
        Dialogs.showState(translate("InputOutputs/Covers", "Writing Cover Informations"),0,allItemNumber, True)
        for rowNo in range(startRowNo,_table.rowCount(),rowStep):
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                if InputOutputs.IA.isWritableFileOrDir(_table.currentTableContentValues[rowNo]["path"]):
                    if _table.isRowHidden(rowNo):
                        InputOutputs.IA.removeFileOrDir(_table.currentTableContentValues[rowNo]["path"])
                        continue
                    baseNameOfDirectory = str(_table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                    baseName = str(_table.currentTableContentValues[rowNo]["baseName"])
                    if _table.isChangableItem(rowNo, 3) or _table.isChangableItem(rowNo, 4):
                        sourcePath = _table.currentTableContentValues[rowNo]["sourceCover"]
                        destinationPath = _table.currentTableContentValues[rowNo]["destinationCover"]
                        if _table.isChangableItem(rowNo, 3):
                            sourcePath = str(_table.item(rowNo,3).text()).strip()
                        if _table.isChangableItem(rowNo, 4):
                            destinationPath = str(_table.item(rowNo,4).text()).strip()
                        if (str(_table.item(rowNo,2).text())!=sourcePath or sourcePath!=destinationPath or str(_table.item(rowNo,2).text())!=destinationPath) or (str(_table.item(rowNo,2).text())!=_table.currentTableContentValues[rowNo]["currentCover"] and (str(_table.item(rowNo,2).text())!=sourcePath and str(_table.item(rowNo,2).text())!=destinationPath)):
                            if str(_table.item(rowNo,3).text()).strip()!="":
                                sourcePath = InputOutputs.IA.getRealPath(sourcePath, _table.currentTableContentValues[rowNo]["path"])
                                if InputOutputs.IA.checkSource(sourcePath, "file"):
                                    if destinationPath!="":
                                        destinationPath = InputOutputs.IA.getRealPath(destinationPath, _table.currentTableContentValues[rowNo]["path"])
                                        if sourcePath!=destinationPath:
                                            destinationPath = InputOutputs.IA.moveOrChange(sourcePath, destinationPath)
                                    else:
                                        destinationPath = sourcePath
                                    InputOutputs.IA.setIconToDirectory(_table.currentTableContentValues[rowNo]["path"], destinationPath)
                                    _table.changedValueNumber += 1
                            else:
                                InputOutputs.IA.setIconToDirectory(_table.currentTableContentValues[rowNo]["path"], "")
                                _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 0, baseNameOfDirectory):
                        baseNameOfDirectory = str(_table.item(rowNo,0).text())
                        _table.changedValueNumber += 1
                    if _table.isChangableItem(rowNo, 1, baseName, False):
                        baseName = str(_table.item(rowNo,1).text())
                        _table.changedValueNumber += 1
                    newFilePath = InputOutputs.getDirName(InputOutputs.getDirName(_table.currentTableContentValues[rowNo]["path"])) + "/" + baseNameOfDirectory + "/" + baseName
                    if InputOutputs.getRealPath(_table.currentTableContentValues[rowNo]["path"]) != InputOutputs.getRealPath(newFilePath):
                        changingFileDirectories.append([_table.currentTableContentValues[rowNo]["path"], 
                                                        newFilePath])
            else:
                allItemNumber = rowNo+1
            Dialogs.showState(translate("InputOutputs/Covers", "Writing Cover Informations"),rowNo+1,allItemNumber, True)
            if isContinueThreadAction==False:
                break
        Universals.finishThreadAction()
        InputOutputs.IA.changeDirectories(changingFileDirectories)
        return True



class CoverTable():
    def __init__(self, _table):
        self.Table = _table
        self.specialTollsBookmarkPointer = "cover"
        self.hiddenTableColumnsSettingKey = "hiddenCoverTableColumns"
        self.refreshColumns()
        pbtnGetFromAmarok = MPushButton(translate("CoverTable", "Get From Amarok"))
        MObject.connect(pbtnGetFromAmarok, SIGNAL("clicked()"), self.getFromAmarok)
        self.Table.hblBox.insertWidget(self.Table.hblBox.count()-1, pbtnGetFromAmarok)
        
    def showDetails(self, _fileNo, _infoNo):
        directoryPathOfCover = self.Table.currentTableContentValues[_fileNo]["path"]
        coverValues = [directoryPathOfCover, 
                       InputOutputs.IA.getRealPath(str(self.Table.item(_fileNo, 2).text()), directoryPathOfCover), 
                       InputOutputs.IA.getRealPath(str(self.Table.item(_fileNo, 3).text()), directoryPathOfCover), 
                       InputOutputs.IA.getRealPath(str(self.Table.item(_fileNo, 4).text()), directoryPathOfCover)]
        CoverDetails.CoverDetails(coverValues, self.Table.isOpenDetailsOnNewWindow.isChecked(), _infoNo)
        
    def cellClicked(self,_row,_column):
        cellLenght = len(self.Table.currentItem().text())*8
        if cellLenght>self.Table.columnWidth(_column):
            self.Table.setColumnWidth(_column,cellLenght)
    
    def cellDoubleClicked(self,_row,_column):
        try:
            if self.Table.tbIsRunOnDoubleClick.isChecked()==True:
                self.showDetails(_row, _column)
        except:
            Dialogs.showError(translate("CoverTable", "Cannot Open File"), 
                        str(translate("CoverTable", "\"%s\" : cannot be opened. Please make sure that you selected a text file.")
                        ) % Organizer.getLink(self.Table.currentTableContentValues[_row]["path"]))
       
    def refreshColumns(self):
        self.Table.tableColumns=[translate("CoverTable", "Directory"), 
                            translate("CoverTable", "Directory Name"), 
                            translate("CoverTable", "Current Cover"), 
                            translate("CoverTable", "Source Cover"), 
                            translate("CoverTable", "Destination Cover")]
        self.Table.tableColumnsKey=["Directory", "Directory Name", "Current Cover", "Source Cover", "Destination Cover"]
        
    def save(self):
        self.Table.checkFileExtensions(4, 3)
        return writeContents(self.Table)
        
    def refresh(self, _path):
        self.Table.currentTableContentValues = readContents(_path)
        self.Table.setRowCount(len(self.Table.currentTableContentValues))
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(5):
                item = None
                if itemNo==0:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"], "directory")
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseNameOfDirectory"])
                elif itemNo==1:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["baseName"], "directory")
                    item = self.Table.createTableWidgetItem(newString, self.Table.currentTableContentValues[rowNo]["baseName"])
                elif itemNo==2:
                    newString = trForUI(self.Table.currentTableContentValues[rowNo]["currentCover"])
                    newString = newString.replace(self.Table.currentTableContentValues[rowNo]["path"], ".")
                    item = self.Table.createTableWidgetItem(newString)
                elif itemNo==3:
                    newString = trForUI(self.Table.currentTableContentValues[rowNo]["sourceCover"])
                    newString = newString.replace(self.Table.currentTableContentValues[rowNo]["path"], ".")
                    oldString = self.Table.currentTableContentValues[rowNo]["currentCover"]
                    oldString = oldString.replace(self.Table.currentTableContentValues[rowNo]["path"], ".")
                    item = self.Table.createTableWidgetItem(newString, oldString)
                elif itemNo==4:
                    newString = Organizer.emend(self.Table.currentTableContentValues[rowNo]["destinationCover"], "file")
                    newString = newString.replace(self.Table.currentTableContentValues[rowNo]["path"], ".")
                    oldString = self.Table.currentTableContentValues[rowNo]["currentCover"]
                    oldString = oldString.replace(self.Table.currentTableContentValues[rowNo]["path"], ".")
                    item = self.Table.createTableWidgetItem(newString, oldString)
                if item!=None:
                    self.Table.setItem(rowNo, itemNo, item)
            if self.Table.currentTableContentValues[rowNo]["isCorrectedFileContent"]==False:
                item = self.Table.item(rowNo, 2)
                if item!=None:
                    item.setBackground(MBrush(MColor(255,163,163)))
                    
    def correctTable(self):
        for rowNo in range(self.Table.rowCount()):
            for itemNo in range(self.Table.columnCount()):
                if itemNo==0 or itemNo==1:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), "directory")
                elif itemNo==2 or itemNo==3:
                    newString = trForUI(str(self.Table.item(rowNo,itemNo).text()))
                else:
                    newString = Organizer.emend(str(self.Table.item(rowNo,itemNo).text()), "file")
                self.Table.item(rowNo,itemNo).setText(trForUI(newString))
        
    def getFromAmarok(self):
        try:
            import Amarok
            Dialogs.showState(translate("CoverTable", "Checking For Amarok..."), 0, 2)
            if Amarok.checkAmarok():
                Dialogs.showState(translate("CoverTable", "Getting Values From Amarok"), 1, 2)
                from Amarok import Operations
                directoriesAndValues = Operations.getDirectoriesAndValues()
                Dialogs.showState(translate("CoverTable", "Values Are Being Processed"), 2, 2)
                if directoriesAndValues!=None:
                    for rowNo in range(self.Table.rowCount()):
                        if self.Table.checkHiddenColumn(3) and self.Table.checkHiddenColumn(4):
                            if _table.isChangableItem(rowNo, 3):
                                directoryPath = InputOutputs.getDirName(InputOutputs.getDirName(_table.currentTableContentValues[rowNo]["path"]))+"/"+str(self.Table.item(rowNo,0).text())+"/"+str(self.Table.item(rowNo,1).text())
                                if directoryPath in directoriesAndValues:
                                    directoryAndValues = directoriesAndValues[directoryPath]
                                    self.Table.item(rowNo,3).setText(directoryAndValues["coverPath"][0].replace(directoryPath, "."))
                                    self.Table.item(rowNo,4).setText("./" + Organizer.getIconName(
                                                            directoryAndValues["artist"][0], 
                                                            directoryAndValues["album"][0], 
                                                            directoryAndValues["genre"][0], 
                                                            directoryAndValues["year"][0]))
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()
        
